// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Caja

#include "caja.h"
#include "tablero.h"

Caja::Caja() {
    for(int i=0; i < 9; i++) {
        cas[i] = NULL;
        for(int j=0; j<3; j++) {
            punteroFila[j][i] = punteroColumna[j][i] = false;
        }
    }
}

void Caja::Iniciar(Tablero &t, int c) {
    for(int i = 0; i < 3; i++)
        for(int j = 0; j < 3; j++)
            cas[i*3+j] = &t.GetCasilla((c/3)*3+i, (c%3)*3+j);
}

bool Caja::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Caja::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

int Caja::CuentaPosibles() {
    int cuenta=0;
    for(int i=0; i<9; i++)
        if(cas[i]->Posible()) cuenta++;
    return cuenta;
}

int Caja::PrimeroPosible() {
    for(int i=0; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

int Caja::SegundoPosible(int p) {
    for(int i=p+1; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

int Caja::TerceroPosible(int s) {
    for(int i=s+1; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

bool Caja::AsignaPunteroFila(int f, int v, int n) {
    if(punteroFila[f][v-1] != n) {
        punteroFila[f][v-1] = n;
        return true;
    }
    return false;
}

bool Caja::AsignaPunteroColumna(int c, int v, int n) {
    if(punteroColumna[c][v-1] != n) {
        punteroColumna[c][v-1] = n;
        return true;
    }
    return false;
}

Unico Caja::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

void Caja::DesmarcaLapiz(int v) {
    for(int p=0; p<9; p++) cas[p]->Lapiz().EliminaMarca(v);
}

Unico Caja::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int c=0; c<9; c++) {
            if(!cas[c]->Valor()) {
                if(cas[c]->Lapiz().ContieneMarca(v)) {
                    u = Unico(c, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Caja::BuscaAla2MenosFila(Casilla *pivote, Casilla *ala1) {
    int f = pivote->Fila();
    MarcasLapiz nocomunes = (pivote->Lapiz() + ala1->Lapiz()) - (pivote->Lapiz() ^ ala1->Lapiz());

    for(int p=0; p<9; p++) {
        if(cas[p]->Fila() != f && cas[p]->Lapiz() == nocomunes) { // Distinta Fila con las dos marcas de lpiz comunes
            return cas[p];
        }
    }
    return NULL;
}

Casilla *Caja::BuscaAla2MenosColumna(Casilla *pivote, Casilla *ala1) {
    int c = pivote->Columna();
    MarcasLapiz nocomunes = (pivote->Lapiz() + ala1->Lapiz()) - (pivote->Lapiz() ^ ala1->Lapiz());

    for(int p=0; p<9; p++) {
        if(cas[p]->Columna() != c && cas[p]->Lapiz() == nocomunes) { // Distinta Fila con las dos marcas de lpiz comunes
            return cas[p];
        }
    }
    return NULL;
}

// Devuelve la caja en la fila fil de la casilla con las marcas de lpiz m
// o -1 si no existe.
int Caja::BuscarMarcas(const MarcasLapiz &m) {

    for(int p=0; p<9; p++) {
        if(cas[p]->Lapiz() == m) return p;
    }
    return -1;
}

//    0    1    2    3    4    5    6    7    8
//   +--  -+-  --+  -XX  X-X  XX-  -XX  X-X  XX-
//   -XX  X-X  XX-  +--  -+-  -+-  -XX  X-X  XX-
//   -XX  X-X  XX-  -XX  X-X  XX-  +--  -+-  --+
//   033  055  066  303  505  606  330  550  660
//
int Caja::RectanguloVacio(int v) {
    int ff[3] = {0, 0, 0};
    int trama[9][3] = {0,3,3, 0,5,5, 0,6,6, 3,0,3, 5,0,5, 6,0,6, 3,3,0, 5,5,0, 6,6,0};
    int retval=-1;

    if(!Presente(v)) {
        for(int p=0; p<9; p++) {
            if(!cas[p]->Lapiz().ContieneMarca(v)) {
                ff[cas[p]->Fila()%3] |= (4 >> (p%3));
            }
        }
        for(int t=0; t<9; t++) {
            if((trama[t][0] == (ff[0] & trama[t][0])) &&
               (trama[t][1] == (ff[1] & trama[t][1])) &&
               (trama[t][2] == (ff[2] & trama[t][2]))) {
                retval = t;
            }
        }
    }
    return retval;
}

Pareja Caja::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int p=0; p<9; p++) {
        if(cas[p]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[p]);
    }
    if(cuenta==2) return par;
    return Pareja();
}
