// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Columna

#include "columna.h"
#include "tablero.h"

void Columna::Iniciar(Tablero &t, int c) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(i, c);
}

bool Columna::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Columna::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

void Columna::CribaMenosCaja(int f) {
    for(int i=0; i<9; i++) if(i/3 != f) cas[i]->MarcaNoPosible();
}

Unico Columna::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

int Columna::UnicoPosible() {
    int cuenta=0;
    int pos = -1;
    for(int i=0; i<9; i++) {
        if(cas[i]->Posible()) {
            cuenta++;
            pos = i;
        }
    }
    if(cuenta == 1) return pos;
    return -1;
}

void Columna::DesmarcaLapiz(int v) {
    for(int f=0; f<9; f++) cas[f]->Lapiz().EliminaMarca(v);
}

void Columna::DesmarcaLapizMenosCaja(int k, int v) {
    for(int i=0; i<9; i++)
        if(i/3 != k) cas[i]->Lapiz().EliminaMarca(v);
}

Unico Columna::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int c=0; c<9; c++) {
            if(!cas[c]->Valor()) {
                if(cas[c]->Lapiz().ContieneMarca(v)) {
                    u = Unico(c, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Columna::BuscaAla1MenosCaja(Casilla *pivote, int f1) {
    int k = pivote->Caja();

    for(int f=f1; f<9; f++) {
        if(cas[f]->Caja() != k && cas[f]->Lapiz().CuentaMarcas() == 2) { // Distinta caja con dos marcas de lpiz
            if((cas[f]->Lapiz() + pivote->Lapiz()).CuentaMarcas() == 3) { // La casilla tiene un candidato en comn con el pivote
                return cas[f];
            }
        }
    }
    return NULL;
}

Casilla *Columna::BuscaAla2MenosCaja(Casilla *pivote, Casilla *ala1) {
    int k = pivote->Caja();
    MarcasLapiz nocomunes = (pivote->Lapiz() + ala1->Lapiz()) - (pivote->Lapiz() ^ ala1->Lapiz());

    for(int f=0; f<9; f++) {
        if(cas[f]->Caja() != k && cas[f]->Lapiz() == nocomunes) { // Distinta caja con las dos marcas de lpiz comunes
            return cas[f];
        }
    }
    return NULL;
}

Pareja Columna::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int f=0; f<9; f++) {
        if(cas[f]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[f]);
    }
    if(cuenta==2) return par;
    return Pareja();
}

Casilla *Columna::DobleCandidatoYMas(Pareja par, int v) {
    bool presA, presB, presC;
    int f1, f2;

    f1 = par.Cas(0)->Fila();
    f2 = par.Cas(1)->Fila();
    // Verificamos si la las columnas de la pareja par tienen el candidato v
    if(cas[f1]->Lapiz().ContieneMarca(v) && cas[f2]->Lapiz().ContieneMarca(v)) {
        // Verificamos si el candidato esta presente en una o dos casillas de la misma fila y caja
        // De cada una de las columnas.
        presA = presB = presC = false;
        for(int f=0; f<9; f++) {
            // Si la casilla contiene el candidato, pero no forma parte del X-Wing
            if(f!=f1 && f!=f2 && cas[f]->Lapiz().ContieneMarca(v)) {
                if(cas[f]->Caja() == cas[f1]->Caja()) presA = true; // Es un ala de par(0)
                else
                if(cas[f]->Caja() == cas[f2]->Caja()) presB = true; // Es un ala de par(1)
                else presC = true; // Presencia en caja diferente
            }
        }
        // Si solo hay alas en par(0) o par(1) retornamos la caja a la que pertenecen el o las alas
        if(presC) return NULL;
        if(presA && !presB) return cas[f1];
        if(!presA && presB) return cas[f2];
    }
    return NULL;
}

int Columna::CuentaCandidato(int v) {
    int cuenta = 0;

    for(int f=0; f<9; f++) {
        if(cas[f]->Lapiz().ContieneMarca(v)) cuenta++;
    }
    return cuenta;
}

// Devuelve la columna en la fila fil de la casilla con las marcas de lpiz m
// o -1 si no existe.
int Columna::BuscarMarcas(const MarcasLapiz &m) {

    for(int f=0; f<9; f++) {
        if(cas[f]->Lapiz() == m) return f;
    }
    return -1;
}

MarcasCasillas Columna::Marcas(int v) {
    MarcasCasillas m;

    for(int f=0; f<9; f++) {
        if(cas[f]->Lapiz().ContieneMarca(v)) m.AsignaCasilla(f);
    }
    return m;
}
