// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Fila

#include "fila.h"
#include "tablero.h"

void Fila::Iniciar(Tablero &t, int f) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(f, i);
}

bool Fila::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Fila::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

void Fila::CribaMenosCaja(int c) {
    for(int i=0; i<9; i++) if(i/3 != c) cas[i]->MarcaNoPosible();
}

Unico Fila::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

int Fila::UnicoPosible() {
    int cuenta=0;
    int pos = -1;
    for(int i=0; i<9; i++) {
        if(cas[i]->Posible()) {
            cuenta++;
            pos = i;
        }
    }
    if(cuenta == 1) return pos;
    return -1;
}

void Fila::DesmarcaLapiz(int v) {
    for(int c=0; c<9; c++) cas[c]->Lapiz().EliminaMarca(v);
}

void Fila::DesmarcaLapizMenosCaja(int k, int v) {
    for(int i=0; i<9; i++)
        if(i/3 != k) cas[i]->Lapiz().EliminaMarca(v);
}

Unico Fila::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int c=0; c<9; c++) {
            if(!cas[c]->Valor()) {
                if(cas[c]->Lapiz().ContieneMarca(v)) {
                    u = Unico(c, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Fila::BuscaAla1MenosCaja(Casilla *pivote, int c1) {
    int k = pivote->Caja();

    for(int c=c1; c<9; c++) {
        if(cas[c]->Caja() != k && cas[c]->Lapiz().CuentaMarcas() == 2) { // Distinta caja con dos marcas de lpiz
            if((cas[c]->Lapiz() + pivote->Lapiz()).CuentaMarcas() == 3) { // La casilla tiene un candidato en comn con el pivote
                return cas[c];
            }
        }
    }
    return NULL;
}

Pareja Fila::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int c=0; c<9; c++) {
        if(cas[c]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[c]);
    }
    if(cuenta==2) return par;
    return Pareja();
}


Casilla *Fila::DobleCandidatoYMas(Pareja par, int v) {
    bool presA, presB, presC;
    int c1, c2;

    c1 = par.Cas(0)->Columna();
    c2 = par.Cas(1)->Columna();
    // Verificamos si la las columnas de la pareja par tienen el candidato v
    if(cas[c1]->Lapiz().ContieneMarca(v) && cas[c2]->Lapiz().ContieneMarca(v)) {
        // Verificamos si el candidato esta presente en una o dos casillas de la misma fila y caja
        // De cada una de las columnas.
        presA = presB = presC = false;
        for(int c=0; c<9; c++) {
            // Si la casilla contiene el candidato, pero no forma parte del X-Wing
            if(c!=c1 && c!=c2 && cas[c]->Lapiz().ContieneMarca(v)) {
                if(cas[c]->Caja() == cas[c1]->Caja()) presA = true; // Es un ala de par(0)
                else
                if(cas[c]->Caja() == cas[c2]->Caja()) presB = true; // Es un ala de par(1)
                else presC = true; // Presencia en caja diferente
            }
        }
        // Si solo hay alas en par(0) o par(1) retornamos la caja a la que pertenecen el o las alas
        if(presC) return NULL;
        if(presA && !presB) return cas[c1];
        if(!presA && presB) return cas[c2];
    }
    return NULL;
}

int Fila::CuentaCandidato(int v) {
    int cuenta = 0;

    for(int c=0; c<9; c++) {
        if(cas[c]->Lapiz().ContieneMarca(v)) cuenta++;
    }
    return cuenta;
}

// Devuelve la columna en la fila fil de la casilla con las marcas de lpiz m
// o -1 si no existe.
int Fila::BuscarMarcas(const MarcasLapiz &m) {

    for(int c=0; c<9; c++) {
        if(cas[c]->Lapiz() == m) return c;
    }
    return -1;
}

MarcasCasillas Fila::Marcas(int v) {
    MarcasCasillas m;

    for(int c=0; c<9; c++) {
        if(cas[c]->Lapiz().ContieneMarca(v)) m.AsignaCasilla(c);
    }
    return m;
}
