// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Grafo

#include "grafo.h"

void Grafo::AsignarAristas() {
    MarcasLapiz marcas;

    // Eliminamos cualquier arista de un nodo consigo mismo
    for(int i=0; i<9; i++) {
        nAristas[i]=0;
        arista[i][i] = false;
    }

    for(int c1=0; c1<8; c1++) {
        for(int c2=c1+1; c2<9; c2++) {
            // Inicialmente eliminamos las aristas
            arista[c1][c2]= arista[c2][c1] = false;
            if(!cas[c1]->Valor() && !cas[c2]->Valor() &&
               cas[c1]->Lapiz().CuentaMarcas() <= grupo &&
               cas[c2]->Lapiz().CuentaMarcas() <= grupo) {
                marcas = cas[c1]->Lapiz() + cas[c2]->Lapiz();
                if(marcas.CuentaMarcas() <= grupo) {
                    nAristas[c1]++;
                    nAristas[c2]++;
                    arista[c1][c2]= arista[c2][c1] = true;
                }
            }
        }
    }
    EliminarAristasUnicas();
}

// Eliminar aristas de nodos con dos aristas menos que grupo (1 para grupos de 3, 2 para grupos de 4)
void Grafo::EliminarAristasUnicas() {
    bool repetir;

    do {
        repetir = false;
        for(int c1=0; c1<9; c1++) {
            if(nAristas[c1]>0 && nAristas[c1]<=grupo-2) {
                repetir = true;
                for(int c2=0; c2<9; c2++) {
                    if(arista[c1][c2]) {
                        nAristas[c1]--;
                        nAristas[c2]--;
                        arista[c1][c2] = arista[c2][c1] = false;
                    }
                }
            }
        }
    } while(repetir);
}

bool Grafo::BuscarGrupo() {
    int nNodos;

    for(int c1=0; c1<9; c1++) {
        for(int c=0; c<9; c++) nodo[c] = false;
        marcas = MarcasLapiz(0);
        nNodos = 0;
        if(nAristas[c1] > 0) {
            nNodos++;
            nodo[c1] = true;
            marcas = marcas + cas[c1]->Lapiz();
            for(int c2=c1+1; c2<9; c2++) {
                if(arista[c1][c2]) {
                    nNodos++;
                    nodo[c2] = true;
                    marcas = marcas + cas[c2]->Lapiz();
                }
            }
            if(nNodos == grupo && marcas.CuentaMarcas() == grupo) {
                for(int i=0,n=0; n<9; n++) if(nodo[n]) nod[i++] = n;
                for(int i=0,v=1; v<=9; v++) if(marcas.ContieneMarca(v)) valor[i++] = v;
                return true;
            }
        }
    }
    return false;
}

void Grafo::EliminaGrupo() {
    for(int i=1; i<=grupo; i++) {
        nAristas[Nodo(i)] = 0;
        for(int j=0; j<9; j++) arista[Nodo(i)][j] = false;
    }
}


int Grafo::Nodo(int n) {
    if(n>grupo) return -1;
    return nod[n-1];
}

int Grafo::Valor(int n) {
    if(n>grupo) return -1;
    return valor[n-1];
}

bool Grafo::Contiene(int p) {
    for(int i=0; i<grupo; i++)
        if(nod[i] == p) return true;
    return false;
}

char *Grafo::Casillas() {
    static char cad[32];

    for(int i=0; i<grupo; i++) {
        cad[i*3] = 'A'+cas[nod[i]]->Fila();
        cad[i*3+1] = '1'+cas[nod[i]]->Columna();
        cad[i*3+2] = ' ';
    }
    cad[3*grupo] = 0;
    return cad;
}

char *Grafo::Valores() {
    static char cad[32];

    for(int i=0; i<grupo; i++) {
        cad[i*2] = '0'+valor[i];
        cad[i*2+1] = ' ';
    }
    cad[2*grupo] = 0;
    return cad;
}
