// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net

#include <stdio.h>
#include <windows.h>
#include "lista.h"
#include "casilla.h"
#include "tablero.h"

char tablero[] = {
    "200080700900360000080010200000500090070003800000000407700400023040008000093007604"
    "000200060800000307040009000000000001104090025090050070000004602000000090005600000" // Hilocometa + XY-Wing
};

// Funcin para mostrar texto en pantalla de consola
void Texto(int f, int c, const char *texto, WORD color) {
    COORD pos;
    DWORD escritos;

    pos.X = c;
    pos.Y = f;
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), color);
    SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), pos);
    WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), texto, strlen(texto), &escritos, NULL);
}

void LogPantalla(char *cad) {
    HANDLE hOut;
    CONSOLE_SCREEN_BUFFER_INFO csbInfo;
    SMALL_RECT sr_ScrollRect = { 0, 42, 50, 52 };// 42, 0, 52, 50
    SMALL_RECT sr_ClipRect = sr_ScrollRect;
    CHAR_INFO chInfo;
    COORD pos = {0, 41};

    chInfo.Attributes = 15;
    chInfo.Char.AsciiChar = ' ';
    hOut = GetStdHandle(STD_OUTPUT_HANDLE);
    GetConsoleScreenBufferInfo(hOut, &csbInfo);
    ScrollConsoleScreenBuffer(hOut, &sr_ScrollRect, &sr_ClipRect, pos, &chInfo);
    Texto(51, 0, cad, 15);
}

// Pausa hasta que se pulse intro
void Pausa() {
    INPUT_RECORD cad[100];
    DWORD leido;

    ReadConsoleInput(GetStdHandle(STD_INPUT_HANDLE), cad, 100, &leido);
    ReadConsoleInput(GetStdHandle(STD_INPUT_HANDLE), cad, 100, &leido);
}

int main()
{
    bool repetir;
    bool insistir;
    char cad[100];

    Tablero T(tablero);
    T.Mostrar();

    T.Soluciones();
    // Si la solucin no es nica, no intentar resolver
    if(T.Cuenta() != 1) {
        if(T.Cuenta() == 0)
            Texto(40, 1, "El sudoku no tiene solucin", 15);
        else
            Texto(40, 1, "El sudoku tiene varias soluciones", 15);
        return 0;
    }

    do {
        repetir=false;
        for(int v=1; v<=9; v++) {
            sprintf(cad, "Criba con valor %d", v);
            Texto(1, 40, cad, 7);
            do {
                T.Criba(v);
                insistir = T.ProcesarCriba(v);
                repetir |= insistir;
                T.Mostrar();
//                Pausa();
            } while(insistir);
        }
        if(!repetir && T.CandidatoUnico()) {
            repetir = true;
            Texto(2,40,"ProcesarCandidatoUnico", 7);
            T.Mostrar();
//            Pausa();
        }
        if(!repetir) {
            for(int v=1; v<=9; v++) {
                sprintf(cad, "Criba con valor %d", v);
                Texto(1, 40, cad, 7);
                T.Criba(v);
                repetir |= T.ProcesarCribaSolitarios(v);
                if(repetir) {
                    Texto(3,40,"ProcesarCribaSolitarios", 7);
                    T.Mostrar();
 //                   Pausa();
                }
            }
        }
        if(!repetir) {
            T.BuscaParesDesnudos();
            if(T.ProcesaParesDesnudos()) {
                repetir = true;
                Texto(4,40,"ProcesaParesDesnudos", 7);
                T.Mostrar();
//                Pausa();
            }
        }
    } while(repetir);

    T.LimpiaCriba();
    T.LimpiaResaltes();

    // FASE 2:
    T.Log("Fase 2");
    T.PonerMarcasLapiz();
    T.Mostrar();
//    Pausa();
    T.VerificarFinales();
    T.Mostrar();
//    Pausa();

    do {
        repetir=false;
        T.BuscaParesDesnudos();
        if(T.ProcesaParesDesnudosFase2()) {
            repetir =true;
            Texto(6,40,"ProcesaParesDesnudos", 7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        T.BuscaPunteros();
        Texto(7,40,"BuscaPunteros",7);
        if(!repetir && T.ProcesaPunteros()) {
            repetir =true;
            Texto(8,40,"ProcesaPunteros",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(2)) {
            repetir =true;
            Texto(9,40,"Grupos de dos",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(3)) {
            repetir =true;
            Texto(10,40,"Grupos de tres",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(4)) {
            repetir =true;
            Texto(11,40,"Grupos de cuatro",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXYWing()) {
            repetir = true;
            Texto(12,40,"Procesa XY-Wing",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXWing()) {
            repetir = true;
            Texto(13,40,"Procesa X-Wing",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.StringKite()) {
            repetir = true;
            Texto(24,40,"Procesa StringKite",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.TodasConDosCandidatosMenosUna()) {
            repetir = true;
            Texto(14,40,"Todas las casillas con dos candidatos menos una",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXWingAlas()) {
            repetir = true;
            Texto(15,40,"Procesa X-Wing Alas",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaSwordFish()) {
            repetir = true;
            Texto(16,40,"Procesa SwordFish",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.SolucionUnicaA()) {
            repetir = true;
            Texto(17,40,"Solucion unica A",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.SolucionUnicaB()) {
            repetir = true;
            Texto(18,40,"Solucion unica B",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXYZWing()) {
            repetir = true;
            Texto(19,40,"Procesa XYZ-Wing",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaSkyscraper()) {
            repetir = true;
            Texto(20,40,"Procesa Skyscraper",7);
            T.Mostrar();
            Pausa();
            T.VerificarFinales();
            T.Mostrar();
            Pausa();
        }
        if(!repetir && T.ProcesaSashimi()) {
            repetir = true;
            Texto(21,40,"Procesa Sashimi X-Wing",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaRectangulosVacios()) {
            repetir = true;
            Texto(22,40,"Rectangulos vacios",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaJellyFish()) {
            repetir = true;
            Texto(23,40,"Procesa Medusa",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.SwordFishAlas()) {
            T.Mostrar();
            Pausa();
            repetir = true;
            Texto(20,40,"Procesa SwordFish alas",7);
            T.VerificarFinales();
            T.Mostrar();
            Pausa();
        }
    } while(repetir);

    T.LimpiaCriba();
    T.LimpiaResaltes();
    T.Mostrar();
    return 0;
}
