// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase MarcasLapiz

#include <cstring>
#include "marcaslapiz.h"

bool MarcasLapiz::AsignaMarca(int v) {
    if(lapiz & 1<<(v-1)) return false;
    lapiz |= 1<<(v-1);
    return true;
}

bool MarcasLapiz::EliminaMarca(int v) {
    if((lapiz & 1<<(v-1)) == 0) return false;
    lapiz &= ~(1<<(v-1));
    return true;
}

int MarcasLapiz::CuentaMarcas() {
    int c=0;
    for(int v=1; v<=9; v++) if(ContieneMarca(v)) c++;
    return c;
}

bool MarcasLapiz::ContieneMarca(int v) {
    return (lapiz & 1<<(v-1));
}

MarcasLapiz MarcasLapiz::operator+(const MarcasLapiz &m1) {
    MarcasLapiz res(0);
    res.lapiz = this->lapiz | m1.lapiz;
    return res;
}

MarcasLapiz MarcasLapiz::operator^(const MarcasLapiz &m1) {
    MarcasLapiz res(0);
    res.lapiz = this->lapiz & m1.lapiz;
    return res;
}

MarcasLapiz MarcasLapiz::operator-(const MarcasLapiz &m1) {
    MarcasLapiz res(0);
    res.lapiz = (this->lapiz | m1.lapiz) & ~(this->lapiz & m1.lapiz);
    return res;
}

bool MarcasLapiz::Quitar(const MarcasLapiz &m1) {
    if(!(lapiz & m1.lapiz)) return false;
    lapiz &= ~m1.lapiz;
    return true;
}

char* MarcasLapiz::Valores() {
    static char cad[20];
    char val[2] = " ";

    std::strcpy(cad, "");
    for(int v=1; v<=9; v++) {
        if(ContieneMarca(v)) {
            val[0] = '0'+v;
            std::strcat(cad, val);
            std::strcat(cad, ",");
        }
    }
    cad[std::strlen(cad)-1]=0;
    return cad;
}

int MarcasLapiz::Valor() {

    if(CuentaMarcas() != 1) return 0;
    for(int v=1; v<=9; v++) if(ContieneMarca(v)) return v;
    return 0;
}

int MarcasLapiz::ValorMarca(int n) {
    for(int v=1; v<=9; v++) {
        if(ContieneMarca(v)) n--;
        if(n == 0) return v;
    }
    return -1;
}
