// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Tablero

#include <stdio.h>
#include <windows.h>
#include "tablero.h"

extern void Texto(int f, int c, const char *texto, WORD color);
extern void LogPantalla(char *cad);

Tablero::Tablero(char* t) : cuenta(0) {
    int f, c;

    for(f=0; f < 9; f++)
        for(c=0; c < 9; c++) {
            casilla[f][c] = Casilla(f, c, t[f*9+c]-'0', t[f*9+c] != '0');
        }
    for(f=0; f < 9; f++) fila[f].Iniciar(*this, f);
    for(c=0; c < 9; c++) columna[c].Iniciar(*this, c);
    for(c=0; c < 9; c++) caja[c].Iniciar(*this, c);
    flog = std::ofstream("solucion.log", std::ios::out);
}

void Tablero::Log(char *cad) {
    flog << cad << std::endl;
    LogPantalla(cad);
}

bool Tablero::Valido(int f, int c, int v) {
    return !fila[f].Presente(v) && !columna[c].Presente(v) && !caja[(f/3)*3+(c/3)].Presente(v);
}

bool Tablero::Soluciones() {
    int f, c, n;
    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(casilla[f][c].Valor() == 0) {
                for(n=1; n<=9; n++) {
                    if(Valido(f, c, n)) {
                        casilla[f][c].Asignar(n);
                        if(Soluciones()) return true;
                        else casilla[f][c].Asignar(0);
                    }
                }
                return false;
            }
        }
    }
    cuenta++;
    return false;
}

void Tablero::Asignar(int f, int c, int v) {
    casilla[f][c].Asignar(v);
    casilla[f][c].Resaltar();
    // Eliminar marcas de lapiz en fila, columna y caja:
    int k = (f/3)*3 + (c/3);
    for(int i=0; i<9; i++) {
        if(fila[f].Cas(i)->Lapiz().EliminaMarca(v)) fila[f].Cas(i)->Resaltar();
        if(columna[c].Cas(i)->Lapiz().EliminaMarca(v)) columna[c].Cas(i)->Resaltar();
        if(caja[k].Cas(i)->Lapiz().EliminaMarca(v)) caja[k].Cas(i)->Resaltar();
    }
}

void Tablero::LimpiaResaltes() {
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].Resaltar(false);
            casilla[f][c].Amarillo(false);
            casilla[f][c].Naranja(false);
        }
    }
}

void Tablero::LimpiaCriba() {
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].MarcaPosible();
        }
    }
}

void Tablero::Criba(int v) {
    int ncaja=0;

    LimpiaCriba();
    LimpiaResaltes();
    // Aplicar criba a valores finales:
    for(int i=0; i<9; i++) {
        if(fila[i].Presente(v)) fila[i].Criba();
        if(columna[i].Presente(v)) columna[i].Criba();
        if(caja[i].Presente(v)) caja[i].Criba();
    }
    // Eliminar de la criba el resto de casillas con valor final
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Valor()) casilla[f][c].MarcaPosible(false);
        }
    }
    // Aplicar criba a parejas y trios punteros de cajas:
    // No desmarcar como posibles las casillas miembros de parejas y trios punteros:
    for(int f=0; f<3; f++) { // Fila de cajas
        for(int c=0; c<3; c++) { // Columna de cajas
            for(int i=0; i<3; i++) { // Fila de punteros
                if(caja[ncaja].PunteroFila(i, v)) fila[f*3+i].CribaMenosCaja(c);
            }
            for(int i=0; i<3; i++) { // Columna de punteros
                if(caja[ncaja].PunteroColumna(i, v)) columna[c*3+i].CribaMenosCaja(f);
            }
            ncaja++;
        }
    }
}

bool Tablero::ProcesarCriba(int v) {
    bool retval = false;
    int p, s, t;
    int f, c;
    char cad[100];

    for(int i=0; i<9; i++) { // Verificar cada caja
        switch(caja[i].CuentaPosibles()) {
        case 1:
            // Final:
            p = caja[i].PrimeroPosible();
            f = caja[i].Cas(p)->Fila();
            c = caja[i].Cas(p)->Columna();
            Asignar(f, c, v);
            sprintf(cad, "Valor final %d en %c%c", v, 'A'+f, '1'+c);
            Log(cad);
            retval=true;
            break;
        case 2:
            // pareja puntero:
            p = caja[i].PrimeroPosible();
            s = caja[i].SegundoPosible(p);
            // Aadir marcas de lpiz y marcarlas en el tablero
            if(caja[i].Cas(p)->Lapiz().AsignaMarca(v))
                caja[i].Cas(p)->Resaltar();
            if(caja[i].Cas(s)->Lapiz().AsignaMarca(v))
                caja[i].Cas(s)->Resaltar();
            // Verificar si p y s estn alineados y guardar pareja puntero si lo estn
            if(caja[i].Cas(p)->Fila() == caja[i].Cas(s)->Fila()) {
                if(caja[i].AsignaPunteroFila(caja[i].Cas(p)->Fila()%3, v, 2)) {
                    sprintf(cad, "Par puntero caja %d fila %c, valor %d", 1+i, 'A'+caja[i].Cas(p)->Fila(), v);
                    Log(cad);
                    retval=true;
                }
            }
            if(caja[i].Cas(p)->Columna() == caja[i].Cas(s)->Columna()) {
                if(caja[i].AsignaPunteroColumna(caja[i].Cas(p)->Columna()%3, v, 2)) {
                    sprintf(cad, "Par puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            break;
        case 3:
            // trio puntero:
            p = caja[i].PrimeroPosible();
            s = caja[i].SegundoPosible(p);
            t = caja[i].TerceroPosible(s);
            // Verificar si p, s y t estn alineados y guardar trie puntero si lo estn
            if(caja[i].Cas(p)->Fila() == caja[i].Cas(s)->Fila() && caja[i].Cas(p)->Fila() == caja[i].Cas(t)->Fila()) {
                if(caja[i].AsignaPunteroFila(caja[i].Cas(p)->Fila()%3, v, 3)) {
                    sprintf(cad, "Trio puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            if(caja[i].Cas(p)->Columna() == caja[i].Cas(s)->Columna() && caja[i].Cas(p)->Columna() == caja[i].Cas(t)->Columna()) {
                if(caja[i].AsignaPunteroColumna(caja[i].Cas(p)->Columna()%3, v, 3)) {
                    sprintf(cad, "Trio puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            break;
        }
    }
    return retval;
}

bool Tablero::CandidatoUnico() {
    bool retval = false;
    Unico u(0,0);
    char cad[100];

    // Buscar candidatos nicos en filas, columnas y cajas:
    for(int f=0; f<9; f++) {
        u = fila[f].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(f, u.Casilla(), u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+f, '1'+u.Casilla());
            Log(cad);
            retval = true;
        }
    }

    for(int c=0; c<9; c++) {
        u = columna[c].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(u.Casilla(), c, u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+u.Casilla(), '1'+c);
            Log(cad);
            retval = true;
        }
    }

    for(int i=0; i<9; i++) {
        u = caja[i].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(caja[i].Cas(u.Casilla())->Fila(), caja[i].Cas(u.Casilla())->Columna(), u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+caja[i].Cas(u.Casilla())->Fila(), '1'+caja[i].Cas(u.Casilla())->Columna());
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::ProcesarCribaSolitarios(int v) {
    bool retval = false;
    int fil, col;
    char cad[100];

    for(int i=0; i<9; i++) { // Verificar cada fila
        col = fila[i].UnicoPosible();
        if(col != -1) {
            Asignar(i, col, v);
            sprintf(cad, "Candidato solitario: Valor final %d en %c%c", v, 'A'+i, '1'+col);
            Log(cad);
            retval = true;
        }
    }

    for(int i=0; i<9; i++) { // Verificar cada columna
        fil = columna[i].UnicoPosible();
        if(fil != -1) {
            Asignar(fil, i, v);
            retval = true;
            sprintf(cad, "Candidato solitario: Valor final %d en %c%c", v, 'A'+fil, '1'+i);
            Log(cad);
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosFila(int f) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        // Solo buscamos en casillas vacas con dos marcas de lpiz.
        if(casilla[f][i].Valor() == 0 && casilla[f][i].Lapiz().CuentaMarcas() == 2) {
            cuenta = coincidencia = 0;
            des.AsignarCas(0, &casilla[f][i]);
            for(int v=1; v<=9; v++) {
                if(casilla[f][i].Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int k=i+1; k<9; k++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(casilla[f][k].Valor() == 0 && casilla[f][k].Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, &casilla[f][k]);
                        for(int v=1; v<=9; v++) {
                            if(casilla[f][k].Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            i++;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosColumna(int c) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        if(casilla[i][c].Valor() == 0 && casilla[i][c].Lapiz().CuentaMarcas() == 2) { // Solo buscamos en casillas vacas
            cuenta = coincidencia = 0;
            des.AsignarCas(0, &casilla[i][c]);
            for(int v=1; v<=9; v++) {
                if(casilla[i][c].Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int k=i+1; k<9; k++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(casilla[k][c].Valor() == 0 && casilla[k][c].Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, &casilla[k][c]);
                        for(int v=1; v<=9; v++) {
                            if(casilla[k][c].Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            i++;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                        break;
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosCaja(int k) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        if(caja[k].Cas(i)->Valor() == 0 && caja[k].Cas(i)->Lapiz().CuentaMarcas() == 2) { // Solo buscamos en casillas vacas
            cuenta = coincidencia = 0;
            des.AsignarCas(0, caja[k].Cas(i));
            for(int v=1; v<=9; v++) {
                if(caja[k].Cas(i)->Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int p=i+1; p<9; p++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(caja[k].Cas(p)->Valor() == 0 && caja[k].Cas(p)->Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, caja[k].Cas(p));
                        for(int v=1; v<=9; v++) {
                            if(caja[k].Cas(p)->Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            retval = true;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudos() {
    bool retval=false;

    // Marcar todos los elementos para borrar:
    for(ListaDesnudo()->Primero(); ListaDesnudo()->Actual(); ListaDesnudo()->Siguiente())
        ListaDesnudo()->ValorActual().MarcarParaBorrar();

    for(int f=0; f<9; f++) {
        retval |= BuscaParesDesnudosFila(f);
    }
    for(int c=0; c<9; c++) {
        retval |= BuscaParesDesnudosColumna(c);
    }
    for(int k=0; k<9; k++) {
        retval |= BuscaParesDesnudosCaja(k);
    }

    // Eliminar elementos marcados para borrar:
    for(ListaDesnudo()->Primero(); ListaDesnudo()->Actual(); ListaDesnudo()->Siguiente())
        if(ListaDesnudo()->ValorActual().MarcadoParaBorrar()) ListaDesnudo()->BorrarActual();

    return retval;
}

bool Tablero::ProcesaParesDesnudosFila(int f) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, fil;
    char cad[100];

    for(int c=0; c<9; c++) {
        caslibre[c] = !casilla[f][c].Valor();
        if(!caslibre[c]) {
            presente[casilla[f][c].Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        fil = ListaDesnudo()->ValorActual().MismaFila();
        if(fil == f) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->Columna()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->Columna()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,c=0; c<9; c++) {
            if(caslibre[c]) libre[i++]=c;
        }
        // En cada casilla libre marcar a lpiz los valores no presentes
        int k0 = casilla[f][libre[0]].Caja();
        int k1 = casilla[f][libre[1]].Caja();

        if(!casilla[f][libre[0]].Valor()) {
            retval |= casilla[f][libre[0]].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f][libre[0]].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[f][libre[1]].Valor()) {
            retval |= casilla[f][libre[1]].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f][libre[1]].Lapiz().AsignaMarca(lapiz[1]);
        }

        if(columna[libre[0]].Presente(lapiz[0]) || caja[k0].Presente(lapiz[0])) {
            Asignar(f, libre[0], lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f, '1'+libre[0]);
            Log(cad);
            retval=true;
        } else
        if(columna[libre[0]].Presente(lapiz[1]) || caja[k0].Presente(lapiz[1])) {
            Asignar(f, libre[0], lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f, '1'+libre[0]);
            Log(cad);
            retval=true;
        }

        if(columna[libre[1]].Presente(lapiz[0]) || caja[k1].Presente(lapiz[0])) {
            Asignar(f, libre[1], lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f, '1'+libre[1]);
            Log(cad);
            retval=true;
        } else
        if(columna[libre[1]].Presente(lapiz[1]) || caja[k1].Presente(lapiz[1])) {
            Asignar(f, libre[1], lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f, '1'+libre[1]);
            Log(cad);
            retval=true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosColumna(int c) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, col;
    char cad[100];

    for(int f=0; f<9; f++) {
        caslibre[f] = !casilla[f][c].Valor();
        if(!caslibre[f]) {
            presente[casilla[f][c].Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        col = ListaDesnudo()->ValorActual().MismaColumna();
        if(col == c) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->Fila()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->Fila()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,f=0; f<9; f++) {
            if(caslibre[f]) libre[i++]=f;
        }
        // En cada casilla libre marcar a lpiz los valores no presentes
        int k0 = casilla[libre[0]][c].Caja();
        int k1 = casilla[libre[1]][c].Caja();

        if(!casilla[libre[0]][c].Valor()) {
            retval |= casilla[libre[0]][c].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[libre[0]][c].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[libre[1]][c].Valor()) {
            retval |= casilla[libre[1]][c].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[libre[1]][c].Lapiz().AsignaMarca(lapiz[1]);
        }

        if(fila[libre[0]].Presente(lapiz[0]) || caja[k0].Presente(lapiz[0])) {
            Asignar(libre[0], c, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+libre[0], '1'+c);
            Log(cad);
            retval=true;
        } else
        if(fila[libre[0]].Presente(lapiz[1]) || caja[k0].Presente(lapiz[1])) {
            Asignar(libre[0], c, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+libre[0], '1'+c);
            Log(cad);
            retval=true;
        }

        if(fila[libre[1]].Presente(lapiz[0]) || caja[k1].Presente(lapiz[0])) {
            Asignar(libre[1], c, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+libre[1], '1'+c);
            Log(cad);
            retval=true;
        } else
        if(fila[libre[1]].Presente(lapiz[1]) || caja[k1].Presente(lapiz[1])) {
            Asignar(libre[1], c, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+libre[1], '1'+c);
            Log(cad);
            retval=true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosCaja(int k) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, caj;
    char cad[100];

    for(int p=0; p<9; p++) {
        caslibre[p] = !caja[k].Cas(p)->Valor();
        if(!caslibre[p]) {
            presente[caja[k].Cas(p)->Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        caj = ListaDesnudo()->ValorActual().MismaCaja();
        if(caj == k) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->PosEnCaja()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->PosEnCaja()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,p=0; p<9; p++) {
            if(caslibre[p]) libre[i++]=p;
        }
        int f0=caja[k].Cas(libre[0])->Fila();
        int f1=caja[k].Cas(libre[1])->Fila();
        int c0=caja[k].Cas(libre[0])->Columna();
        int c1=caja[k].Cas(libre[1])->Columna();

        if(!casilla[f0][c0].Valor()) {
            retval |= casilla[f0][c0].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f0][c0].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[f1][c1].Valor()) {
            retval |= casilla[f1][c1].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f1][c1].Lapiz().AsignaMarca(lapiz[1]);
        }

        // En cada casilla libre marcar a lpiz los valores no presentes
        if(fila[f0].Presente(lapiz[0]) || columna[c0].Presente(lapiz[0])) {
            Asignar(f0, c0, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f0, '1'+c0);
            Log(cad);
            retval = true;
        } else
        if(fila[f0].Presente(lapiz[1]) || columna[c0].Presente(lapiz[1])) {
            Asignar(f0, c0, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f0, '1'+c0);
            Log(cad);
            retval = true;
        }
        if(fila[f1].Presente(lapiz[0]) || columna[c1].Presente(lapiz[0])) {
            Asignar(f1, c1, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f1, '1'+c1);
            Log(cad);
            retval = true;
        } else
        if(fila[f1].Presente(lapiz[1]) || columna[c1].Presente(lapiz[1])) {
            Asignar(f1, c1, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f1, '1'+c1);
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudos() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaParesDesnudosFila(f);
    for(int c=0; c<9; c++) retval |= ProcesaParesDesnudosColumna(c);
    for(int k=0; k<9; k++) retval |= ProcesaParesDesnudosCaja(k);
    return retval;
}

// Procedimientos de fase 2:
void Tablero::PonerMarcasLapiz() {
    // Para cada casilla colocar marcas de lpiz que faltan:
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(!casilla[f][c].Valor()) { // Si no tiene un valor asignado
                for(int v=1; v<=9; v++) {
                    int k = casilla[f][c].Caja();
                    if(!fila[f].Presente(v) && !columna[c].Presente(v) && !caja[k].Presente(v))
                        casilla[f][c].Lapiz().AsignaMarca(v);
                }
            }
        }
    }
    for(int v=1; v<=9; v++) {
        int ncaja=0;
        for(int f=0; f<3; f++) { // Fila de cajas
            for(int c=0; c<3; c++) { // Columna de cajas
                for(int i=0; i<3; i++) { // Fila de punteros
                    if(caja[ncaja].PunteroFila(i, v)) fila[f*3+i].DesmarcaLapizMenosCaja(c, v);
                }
                for(int i=0; i<3; i++) { // Columna de punteros
                    if(caja[ncaja].PunteroColumna(i, v)) columna[c*3+i].DesmarcaLapizMenosCaja(f, v);
                }
                ncaja++;
            }
        }
    }
}

bool Tablero::CasillasConUnaMarcaLapiz() {
    bool retval=false;
    int v;
    char cad[100];

    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() == 1) {
                retval = true;
                v=1;
                while(!casilla[f][c].Lapiz().ContieneMarca(v)) v++;
                Asignar(f, c, v);
                sprintf(cad, "Unica Marca Lapiz %c%c, valor %d", 'A'+f, '1'+c, v);
                Log(cad);
            }
        }
    }
    return retval;
}

bool Tablero::UnicaCasillaConCandidato() {
    bool retval = false;
    Unico u(0,0);
    char cad[100];

    // Buscar casillas con nico valor de candidato en filas, columnas y cajas:
    for(int f=0; f<9; f++) {
        u = fila[f].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            Asignar(f, u.Casilla(), u.Valor());
            sprintf(cad, "Unica Casilla Candidato Fila %c%c, valor %d", 'A'+f, '1'+u.Casilla(), u.Valor());
            Log(cad);
            retval = true;
        }
    }

    for(int c=0; c<9; c++) {
        u = columna[c].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            Asignar(u.Casilla(), c, u.Valor());
            sprintf(cad, "Unica Casilla Candidato Columna %c%c, valor %d", 'A'+u.Casilla(), '1'+c, u.Valor());
            Log(cad);
            retval = true;
        }
    }

    for(int k=0; k<9; k++) {
        u = caja[k].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            int fil = caja[k].Cas(u.Casilla())->Fila();
            int col = caja[k].Cas(u.Casilla())->Columna();
            Asignar(fil, col, u.Valor());
            sprintf(cad, "Unica Casilla Candidato Caja %d %c%c, valor %d", k, 'A'+fil, '1'+col, u.Valor());
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::VerificarFinales() {
    bool retval=false;

    while(CasillasConUnaMarcaLapiz()) {
        Texto(5,40,"CasillasConUnaMarcaLapiz",7);
        retval = true;
    }
    while(UnicaCasillaConCandidato()) {
        Texto(6,40,"UnicaCasillaConCandidato",7);
        retval = true;
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosFilaFase2(int f) {
    bool retval = false;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int fil = ListaDesnudo()->ValorActual().MismaFila();
        if(fil == f) {
            for(int c=0; c<9; c++) {
                if(c != ListaDesnudo()->ValorActual().Cas(0)->Columna() &&
                   c != ListaDesnudo()->ValorActual().Cas(1)->Columna() &&
                   (!ListaDesnudo()->ValorActual().Trio() || c!= ListaDesnudo()->ValorActual().Cas(2)->Columna())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    };
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosColumnaFase2(int c) {
    bool retval = false;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int col = ListaDesnudo()->ValorActual().MismaColumna();
        if(col == c) {
            for(int f=0; f<9; f++) {
                if(f != ListaDesnudo()->ValorActual().Cas(0)->Fila() &&
                   f != ListaDesnudo()->ValorActual().Cas(1)->Fila()&&
                   (!ListaDesnudo()->ValorActual().Trio() || f!= ListaDesnudo()->ValorActual().Cas(2)->Fila())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    }
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosCajaFase2(int k) {
    bool retval = false;
    int f, c;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int caj = ListaDesnudo()->ValorActual().MismaCaja();
        if(caj == k) {
            for(int p=0; p<9; p++) {
                if(p != ListaDesnudo()->ValorActual().Cas(0)->PosEnCaja() &&
                   p != ListaDesnudo()->ValorActual().Cas(1)->PosEnCaja() &&
                   (!ListaDesnudo()->ValorActual().Trio() || p!= ListaDesnudo()->ValorActual().Cas(2)->PosEnCaja())) {
                    f = caja[k].Cas(p)->Fila();
                    c = caja[k].Cas(p)->Columna();
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    }
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosFase2() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaParesDesnudosFilaFase2(f);
    for(int c=0; c<9; c++) retval |= ProcesaParesDesnudosColumnaFase2(c);
    for(int k=0; k<9; k++) retval |= ProcesaParesDesnudosCajaFase2(k);
    return retval;
}

// Buscar pares y trios puntero
bool Tablero::BuscaPunteros() {
    bool retval = false;

    for(int v=1; v<=9; v++) {
        for(int f=0; f<9; f++)
            retval |= BuscaPunterosFila(f, v);
        for(int c=0; c<9; c++)
            retval |= BuscaPunterosColumna(c, v);
        for(int k=0; k<9; k++)
            retval |= BuscaPunterosCaja(k, v);
    }
    return retval;
}

// Busca el candidato v en la fila f, si est dos o tres veces en la misma caja lo guarda.
bool Tablero::BuscaPunterosFila(int f, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!fila[f].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int c=0; c<9; c++) if(fila[f].Cas(c)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int c=0; c<9; c++) {
                if(fila[f].Cas(c)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, fila[f].Cas(c));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaCaja()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::BuscaPunterosColumna(int c, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!columna[c].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int f=0; f<9; f++) if(columna[c].Cas(f)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int f=0; f<9; f++) {
                if(columna[c].Cas(f)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, columna[c].Cas(f));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaCaja()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::BuscaPunterosCaja(int k, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!caja[k].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int p=0; p<9; p++) if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int p=0; p<9; p++) {
                if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, caja[k].Cas(p));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaFila()!=-1 || pun.MismaColumna()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::ProcesaPunterosFila(int f) {
    bool retval = false;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int fil = ListaPuntero()->ValorActual().MismaFila();
        if(fil == f) {
            for(int c=0; c<9; c++) { // Comparar casillas?? casilla[f][c] != ListaPuntero()->ValorActual().Cas(0)
                if(c != ListaPuntero()->ValorActual().Cas(0)->Columna() &&
                   c != ListaPuntero()->ValorActual().Cas(1)->Columna() &&
                   (!ListaPuntero()->ValorActual().Trio() || c!= ListaPuntero()->ValorActual().Cas(2)->Columna())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunterosColumna(int c) {
    bool retval = false;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int col = ListaPuntero()->ValorActual().MismaColumna();
        if(col == c) {
            for(int f=0; f<9; f++) {
                if(f != ListaPuntero()->ValorActual().Cas(0)->Fila() &&
                   f != ListaPuntero()->ValorActual().Cas(1)->Fila()&&
                   (!ListaPuntero()->ValorActual().Trio() || f!= ListaPuntero()->ValorActual().Cas(2)->Fila())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunterosCaja(int k) {
    bool retval = false;
    int f, c;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int caj = ListaPuntero()->ValorActual().MismaCaja();
        if(caj == k) {
            for(int p=0; p<9; p++) {
                if(p != ListaPuntero()->ValorActual().Cas(0)->PosEnCaja() &&
                   p != ListaPuntero()->ValorActual().Cas(1)->PosEnCaja() &&
                   (!ListaPuntero()->ValorActual().Trio() || p!= ListaPuntero()->ValorActual().Cas(2)->PosEnCaja())) {
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        f = caja[k].Cas(p)->Fila();
                        c = caja[k].Cas(p)->Columna();
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunteros() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaPunterosFila(f);
    for(int c=0; c<9; c++) retval |= ProcesaPunterosColumna(c);
    for(int k=0; k<9; k++) retval |= ProcesaPunterosCaja(k);
    return retval;
}

bool Tablero::ProcesaGrafoFila(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    // Para cada fila:
    for(int f=0; f<9; f++) {
        g.IniciarMarcas(fila[f].GetCasillas());
        g.AsignarAristas();
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de fila:
            for(int c=0; c<9; c++) {
                if(!g.Contiene(c)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= fila[f].Cas(c)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int c=0; c<9; c++) {
                    if(!g.Contiene(c)) {
                        for(int i=1; i<=grupo; i++) {
                            if(fila[f].Cas(c)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+f, '1'+c);
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafoColumna(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    for(int c=0; c<9; c++) {
        g.IniciarMarcas(columna[c].GetCasillas());
        g.AsignarAristas();
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de columna:
            for(int f=0; f<9; f++) {
                if(!g.Contiene(f)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= columna[c].Cas(f)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int f=0; f<9; f++) {
                    if(!g.Contiene(f)) {
                        for(int i=1; i<=grupo; i++) {
                            if(columna[c].Cas(f)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+f, '1'+c);
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafoCaja(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    for(int k=0; k<9; k++) {
        g.IniciarMarcas(caja[k].GetCasillas());
        g.AsignarAristas();
        // Para cada grupo existente
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de caja:
            for(int p=0; p<9; p++) {
                if(!g.Contiene(p)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int p=0; p<9; p++) {
                    if(!g.Contiene(p)) {
                        for(int i=1; i<=grupo; i++) {
                            if(caja[k].Cas(p)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafo(int grupo) {
    bool retval = false;

    retval |= ProcesaGrafoFila(grupo);
    retval |= ProcesaGrafoColumna(grupo);
    retval |= ProcesaGrafoCaja(grupo);

    return retval;
}

bool Tablero::ProcesaXYWing() {
    int f, c;
    Casilla *pivote;
    Casilla *ala1;
    Casilla *ala2;
    int c1, f1;

    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() == 2) { // Posible pivote
                pivote = &casilla[f][c];
                c1 = 0;
                do {
                    ala1 = fila[f].BuscaAla1MenosCaja(pivote, c1);
                    if(ala1) { // Tenemos posible ala 1 en f,c2 Casos 1/2
                        c1 = ala1->Columna()+1; // Probar si hay ms candidatos a ala1
                        // Buscar ala2
                        ala2 = columna[c].BuscaAla2MenosCaja(pivote, ala1);
                        if(ala2) { // Patrn XY-Wing caso 1
                            if(ProcesarXYWingCaso1(pivote, ala1, ala2)) return true;
                        } else { // Posible caso 2
                            ala2 = caja[pivote->Caja()].BuscaAla2MenosFila(pivote, ala1);
                            if(ala2) { // Patrn XY-Wing caso 2
                                if(ProcesarXYWingCaso2(pivote, ala1, ala2)) return true;
                            }
                        }
                    } else { // Posible caso 3
                        c1 = 9; // No hemos encontrado el ala1 en la fila f. No seguir buscando.
                        f1 = 0;
                        do {
                            ala1 = columna[c].BuscaAla1MenosCaja(pivote, f1);
                            if(ala1) { // Tenemos posible ala 1 en f2,c
                                f1 = ala1->Fila()+1;
                                ala2 = caja[pivote->Caja()].BuscaAla2MenosColumna(pivote, ala1);
                                if(ala2) { // Patrn XY-Wing caso 3
                                    if(ProcesarXYWingCaso3(pivote, ala1, ala2)) return true;
                                }
                            }
                        } while(ala1 && f1 < 9); // Probar si hay ms candidatos a ala1
                    }
                } while(ala1 && c1 < 9);
            }
        }
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso1(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    char cad[100];

    if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 1 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala2->Fila(), '1'+ala1->Columna());
            Log(cad);
            casilla[ala2->Fila()][ala1->Columna()].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso2(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    int c1, c2, c3;
    int k = pivote->Caja();
    char cad[100];

    c1 = caja[k].Cas(0)->Columna();
    c2 = caja[k].Cas(1)->Columna();
    c3 = caja[k].Cas(2)->Columna();
    if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c1].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c2].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c3].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 2 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala2->Fila(), '1'+ala1->Columna());
            Log(cad);
            casilla[ala2->Fila()][ala1->Columna()].Resaltar();
        }
        if(casilla[pivote->Fila()][c1].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c1);
            Log(cad);
            casilla[pivote->Fila()][c1].Resaltar();
        }
        if(casilla[pivote->Fila()][c2].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c2);
            Log(cad);
            casilla[pivote->Fila()][c2].Resaltar();
        }
        if(casilla[pivote->Fila()][c3].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c3);
            Log(cad);
            casilla[pivote->Fila()][c3].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso3(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    int f1, f2, f3;
    int k = pivote->Caja();
    char cad[100];

    f1 = caja[k].Cas(0)->Fila();
    f2 = caja[k].Cas(3)->Fila();
    f3 = caja[k].Cas(6)->Fila();
    if(casilla[ala1->Fila()][ala2->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f1][pivote->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f2][pivote->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f3][pivote->Columna()].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 3 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala1->Fila()][ala2->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala1->Fila(), '1'+ala2->Columna());
            Log(cad);
            casilla[ala1->Fila()][ala2->Columna()].Resaltar();
        }
        if(casilla[f1][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f1, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        if(casilla[f2][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f2, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        if(casilla[f3][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f3, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesaXWing() {
    Pareja par1, par2;

    LimpiaResaltes();

    for(int v=1; v<=9; v++) { // Para cada valor
        // Empezamos con las filas. No buscaremos en la ltima,
        // ya que no podemos comparar con las siguientes
        for(int f=0; f<8; f++) {
            // Detectar fila con dos casillas con el candidato v
            par1 = fila[f].DobleCandidato(v);
            // Si par1(0) != NULL, tenemos un candidato a X-Wing, buscaremos en el resto de filas
            if(par1.Cas(0) != NULL) {
                for(int f2=f+1; f2<9; f2++) {
                    par2 = fila[f2].DobleCandidato(v);
                    if(par2.Cas(0) != NULL &&
                       par1.Cas(0)->Columna() == par2.Cas(0)->Columna() &&
                       par1.Cas(1)->Columna() == par2.Cas(1)->Columna()) {
                        if(ProcesaXWingFila(par1, par2, v)) return true;
                    }
                }
            }
        }
        // Y ahora las columnas.
        for(int c=0; c<8; c++) {
            // Detectar fila con dos casillas con el candidato v
            par1 = columna[c].DobleCandidato(v);
            // Si par1(0) != NULL, tenemos un candidato a X-Wing, buscaremos en el resto de columnas
            if(par1.Cas(0) != NULL) {
                for(int c2=c+1; c2<9; c2++) {
                    par2 = columna[c2].DobleCandidato(v);
                    if(par2.Cas(0) != NULL &&
                       par1.Cas(0)->Fila() == par2.Cas(0)->Fila() &&
                       par1.Cas(1)->Fila() == par2.Cas(1)->Fila()) {
                        if(ProcesaXWingColumna(par1, par2, v)) return true;
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::ProcesaXWingFila(Pareja a, Pareja b, int v) {
    bool retval=false;
    char cad[100];

    for(int f=0; f<9; f++) {
        if(f != a.Cas(0)->Fila() && f != b.Cas(0)->Fila()) {
            retval |= casilla[f][a.Cas(0)->Columna()].Lapiz().ContieneMarca(v);
            retval |= casilla[f][a.Cas(1)->Columna()].Lapiz().ContieneMarca(v);
        }
    }
    if(retval) {
        sprintf(cad, "X-Wing %c%c %c%c %c%c %c%c valor %d",
            'A'+a.Cas(0)->Fila(), '1'+a.Cas(0)->Columna(),
            'A'+a.Cas(1)->Fila(), '1'+a.Cas(1)->Columna(),
            'A'+b.Cas(0)->Fila(), '1'+b.Cas(0)->Columna(),
            'A'+b.Cas(1)->Fila(), '1'+b.Cas(1)->Columna(), v);
        Log(cad);
        a.Cas(0)->Amarillo();
        a.Cas(1)->Amarillo();
        b.Cas(0)->Amarillo();
        b.Cas(1)->Amarillo();
        for(int f=0; f<9; f++) {
            if(f != a.Cas(0)->Fila() && f != b.Cas(0)->Fila()) {
                if(casilla[f][a.Cas(0)->Columna()].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+f, '1'+a.Cas(0)->Columna());
                    Log(cad);
                    casilla[f][a.Cas(0)->Columna()].Resaltar();
                }
                if(casilla[f][a.Cas(1)->Columna()].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+f, '1'+a.Cas(1)->Columna());
                    Log(cad);
                    casilla[f][a.Cas(1)->Columna()].Resaltar();
                }
            }
        }
    }

    return retval;
}

bool Tablero::ProcesaXWingColumna(Pareja a, Pareja b, int v) {
    bool retval=false;
    char cad[100];

    for(int c=0; c<9; c++) {
        if(c != a.Cas(0)->Columna() && c != b.Cas(0)->Columna()) {
            retval |= casilla[a.Cas(0)->Fila()][c].Lapiz().ContieneMarca(v);
            retval |= casilla[a.Cas(1)->Fila()][c].Lapiz().ContieneMarca(v);
        }
    }
    if(retval) {
        sprintf(cad, "X-Wing %c%c %c%c %c%c %c%c valor %d",
            'A'+a.Cas(0)->Fila(), '1'+a.Cas(0)->Columna(),
            'A'+a.Cas(1)->Fila(), '1'+a.Cas(1)->Columna(),
            'A'+b.Cas(0)->Fila(), '1'+b.Cas(0)->Columna(),
            'A'+b.Cas(1)->Fila(), '1'+b.Cas(1)->Columna(), v);
        Log(cad);
        a.Cas(0)->Amarillo();
        a.Cas(1)->Amarillo();
        b.Cas(0)->Amarillo();
        b.Cas(1)->Amarillo();
        for(int c=0; c<9; c++) {
            if(c != a.Cas(0)->Columna() && c != b.Cas(0)->Columna()) {
                if(casilla[a.Cas(0)->Fila()][c].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+a.Cas(0)->Fila(), '1'+c);
                    Log(cad);
                    casilla[a.Cas(0)->Fila()][c].Resaltar();
                }
                if(casilla[a.Cas(1)->Fila()][c].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+a.Cas(1)->Fila(), '1'+c);
                    Log(cad);
                    casilla[a.Cas(1)->Fila()][c].Resaltar();
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaXWingAlasFilas(int v) {
    bool retval = false;
    Pareja par;

    for(int f=0; f<9; f++) {
        // Detectar fila con dos casillas con el candidato v
        par = fila[f].DobleCandidato(v);
        // Si par.Cas(0) != NULL, y la pareja no est en la misma caja
        // tenemos un candidato a X-Wing con alas, buscaremos en el resto de filas
        if(par.Cas(0) != NULL && !par.MismaCaja()) {
            for(int f2=0; f2<9; f2++) {
                // No nos interesa que la segunda fila est en la misma fila de cajas que la primera
                if(f/3 != f2/3) {
                    Casilla *c2 = fila[f2].DobleCandidatoYMas(par, v);
                    if(c2) {
                        retval |= ProcesaXWingAlasFilas(par, c2, v);
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaXWingAlasColumnas(int v) {
    bool retval = false;
    Pareja par;

    for(int c=0; c<9; c++) {
        // Detectar columna con dos casillas con el candidato v
        par = columna[c].DobleCandidato(v);
        // Si par.Cas(0) != NULL, y la pareja no est en la misma caja
        // tenemos un candidato a X-Wing con alas, buscaremos en el resto de columnas
        if(par.Cas(0) != NULL && !par.MismaCaja()) {
            for(int c2=0; c2<9; c2++) {
                // No nos interesa que la segunda columna est en la misma columna de cajas que la primera
                if(c/3 != c2/3) {
                    Casilla *f2 = columna[c2].DobleCandidatoYMas(par, v);
                    if(f2) {
                        retval |= ProcesaXWingAlasColumnas(par, f2, v);
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaXWingAlasFilas(Pareja par, Casilla *c2, int v) {
    bool retval = false;
    char cad[100];

    // Verificar si el candidato est presente en la caja y columna de c2, salvo en c2
    for(int f=0; f<9; f++) {
        retval |= columna[c2->Columna()].Cas(f) != c2 &&                    // No mirar en la casilla c2
                  columna[c2->Columna()].Cas(f)->Caja() == c2->Caja() &&    // En la misma caja que c2
                  columna[c2->Columna()].Cas(f)->Lapiz().ContieneMarca(v);  // Y contiene el candidato v
    }
    if(retval) {
        sprintf(cad, "X-Wing Alas %c%c %c%c %c%c %c%c Valor %d",
                'A'+par.Cas(0)->Fila(), '1'+par.Cas(0)->Columna(),
                'A'+par.Cas(1)->Fila(), '1'+par.Cas(1)->Columna(),
                'A'+c2->Fila(), '1'+par.Cas(0)->Columna(),
                'A'+c2->Fila(), '1'+par.Cas(1)->Columna(), v);
        Log(cad);
        par.Cas(0)->Amarillo();
        par.Cas(1)->Amarillo();
        casilla[c2->Fila()][par.Cas(0)->Columna()].Amarillo();
        casilla[c2->Fila()][par.Cas(1)->Columna()].Amarillo();
        // Eliminar el candidato si est presente en la caja y columna de c2, salvo en c2
        for(int f=0; f<9; f++) {
            if(columna[c2->Columna()].Cas(f) != c2 &&
               columna[c2->Columna()].Cas(f)->Caja() == c2->Caja() &&
               columna[c2->Columna()].Cas(f)->Lapiz().EliminaMarca(v))
                casilla[f][c2->Columna()].Resaltar();
        }

        // Resaltar las alas:
        for(int c=0; c<9; c++) {
            if(fila[c2->Fila()].Cas(c) != c2 &&
               fila[c2->Fila()].Cas(c)->Caja() == c2->Caja() &&
               fila[c2->Fila()].Cas(c)->Lapiz().ContieneMarca(v))
                casilla[c2->Fila()][c].Naranja();
        }
    }
    return retval;
}

bool Tablero::ProcesaXWingAlasColumnas(Pareja par, Casilla *f2, int v) {
    bool retval = false;
    char cad[100];

    // Verificar si el candidato est presente en la caja y fila de f2, salvo en f2
    for(int c=0; c<9; c++) {
        retval |= fila[f2->Fila()].Cas(c) != f2 &&                    // No mirar en la casilla f2
                  fila[f2->Fila()].Cas(c)->Caja() == f2->Caja() &&    // En la misma caja que f2
                  fila[f2->Fila()].Cas(c)->Lapiz().ContieneMarca(v);  // Y contiene el candidato v
    }
    if(retval) {
        sprintf(cad, "X-Wing Alas %c%c %c%c %c%c %c%c Valor %d",
                'A'+par.Cas(0)->Fila(), '1'+par.Cas(0)->Columna(),
                'A'+par.Cas(1)->Fila(), '1'+par.Cas(1)->Columna(),
                'A'+par.Cas(0)->Fila(), '1'+f2->Columna(),
                'A'+par.Cas(1)->Fila(), '1'+f2->Columna(), v);
        Log(cad);
        par.Cas(0)->Amarillo();
        par.Cas(1)->Amarillo();
        casilla[par.Cas(0)->Fila()][f2->Columna()].Amarillo();
        casilla[par.Cas(1)->Fila()][f2->Columna()].Amarillo();
        // Eliminar el candidato si est presente en la caja y fila de f2, salvo en f2
        for(int c=0; c<9; c++) {
            if(fila[f2->Fila()].Cas(c) != f2 &&
               fila[f2->Fila()].Cas(c)->Caja() == f2->Caja() &&
               fila[f2->Fila()].Cas(c)->Lapiz().EliminaMarca(v))
                casilla[f2->Fila()][c].Resaltar();
        }

        // Resaltar las alas:
        for(int f=0; f<9; f++) {
            if(columna[f2->Fila()].Cas(f) != f2 &&
               columna[f2->Fila()].Cas(f)->Caja() == f2->Caja() &&
               columna[f2->Fila()].Cas(f)->Lapiz().ContieneMarca(v))
                casilla[f][f2->Columna()].Naranja();
        }
    }
    return retval;
}

bool Tablero::ProcesaXWingAlas() {
    bool retval = false;

    LimpiaResaltes();

    for(int v=1; v<=9; v++) {
        // Empezamos con las filas.
        retval |= BuscaXWingAlasFilas(v);
        // Y ahora las columnas.
        retval |= BuscaXWingAlasColumnas(v);
    }
    return retval;
}

bool Tablero::BuscaSwordFishFilas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;


    for(int f1=0; f1<7; f1++) {
        // Buscamos en las primeras 7 filas en las que no aparezca v
        // y est como candidato en 2, 3 columnas:
        if(!fila[f1].Presente(v) && fila[f1].CuentaCandidato(v) <= 3) {
            // Ahora buscamos las filas f1+1 a septima otra que tenga 2, 3 candidatos v.
            for(int f2= f1+1; f2<8; f2++) {
                if(!fila[f2].Presente(v) && fila[f2].CuentaCandidato(v) <= 3) {
                    // Si entre las dos filas, el candidato aparece en 2, 3 columnas, buscamos otra fila
                    CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v);
                    if(CasillasColumnas.CuentaCasillas() == 3) {
                        // Ahora buscamos las filas f2+1 a novena otra que tenga 2, 3 candidatos v en las
                        // mismas columnas que las anteriores.
                        for(int f3=f2+1; f3<9; f3++) {
                            if(!fila[f3].Presente(v) && fila[f3].CuentaCandidato(v) <= 3) {
                                CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v)+fila[f3].Marcas(v);
                                if(CasillasColumnas.CuentaCasillas() == 3) {
                                    // Tenemos una medusa.
                                    CasillasFilas.Borra();
                                    CasillasFilas.AsignaCasilla(f1);
                                    CasillasFilas.AsignaCasilla(f2);
                                    CasillasFilas.AsignaCasilla(f3);
                                    if(ProcesaSwordFishFilas(CasillasFilas, CasillasColumnas, v)) return true;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaSwordFishColumnas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;

    for(int c1=0; c1<7; c1++) {
        // Buscamos en las primeras 6 columnas en las que no aparezca v
        // y est como candidato en 2 o 3 columnas:
        if(!columna[c1].Presente(v) && columna[c1].CuentaCandidato(v) <= 3) {
            // Ahora buscamos las columnas c1+1 a septima otra que tenga 2, 3 o 4 candidatos v.
            for(int c2= c1+1; c2<8; c2++) {
                if(!columna[c2].Presente(v) && columna[c2].CuentaCandidato(v) <= 3) {
                    // Si entre las dos columnas, el candidato aparece en tres filas, buscamos otra columna
                    CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v);
                    if(CasillasFilas.CuentaCasillas() == 3) {
                        // Ahora buscamos las columnas c2+1 a octava otra que tenga 2, 3 o 4 candidatos v en las
                        // mismas filas que las anteriores.
                        for(int c3=c2+1; c3<9; c3++) {
                            if(!columna[c3].Presente(v) && columna[c3].CuentaCandidato(v) <= 3) {
                                CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v)+columna[c3].Marcas(v);
                                if(CasillasFilas.CuentaCasillas() == 3) {
                                    CasillasColumnas.Borra();
                                    CasillasColumnas.AsignaCasilla(c1);
                                    CasillasColumnas.AsignaCasilla(c2);
                                    CasillasColumnas.AsignaCasilla(c3);
                                    if(ProcesaSwordFishColumnas(CasillasFilas, CasillasColumnas, v)) return true;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaSwordFishFilas(MarcasCasillas fils, MarcasCasillas cols, int v) {
    bool retval = false;
    char cad[100];
    int ff[3], i=0;
    int cc[3], j=0;

    for(int f=0; f<9; f++) {
        if(!fils.ContieneCasilla(f)) {
            for(int c=0; c<9; c++) {
                if(cols.ContieneCasilla(c)) {
                    retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                }
            }
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(fils.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(cols.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Pez espada F: fila [%c,%c,%c] col [%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], '1'+cc[0], '1'+cc[1], '1'+cc[2], v);
        Log(cad);
        for(i=0; i<3; i++) {
            for(j=0; j<3; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }
        if(casilla[ff[0]])
        for(int f=0; f<9; f++) {
            if(!fils.ContieneCasilla(f)) {
                for(int c=0; c<9; c++) {
                    if(cols.ContieneCasilla(c)) {
                        if(casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Elimina candidato %d en %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaSwordFishColumnas(MarcasCasillas fils, MarcasCasillas cols, int v) {
    bool retval = false;
    char cad[100];
    int ff[3], i=0;
    int cc[3], j=0;

    for(int c=0; c<9; c++) {
        if(!cols.ContieneCasilla(c)) {
            for(int f=0; f<9; f++) {
                if(fils.ContieneCasilla(f)) {
                    retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                }
            }
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(fils.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(cols.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Pez espada C: fila [%c,%c,%c] col [%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], '1'+cc[0], '1'+cc[1], '1'+cc[2], v);
        Log(cad);
        for(i=0; i<3; i++) {
            for(j=0; j<3; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }
        if(casilla[cc[0]])
        for(int c=0; c<9; c++) {
            if(!cols.ContieneCasilla(c)) {
                for(int f=0; f<9; f++) {
                    if(fils.ContieneCasilla(f)) {
                        if(casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Elimina candidato %d en %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaSwordFish() {
    bool retval=false;

    LimpiaResaltes();
    for(int v=1; v<=9; v++) {
        retval |= BuscaSwordFishFilas(v);
        retval |= BuscaSwordFishColumnas(v);
    }
    return retval;
}

bool Tablero::TodasConDosCandidatosMenosUna() {
    int cuenta3Candidatos=0;
    Casilla *cas3;
    char cad[100];

    LimpiaResaltes();
    // Buscar en cada casilla, si el nmero de candidatos en ms de una es tres
    // o en alguna hay ms de tres, retornar false
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() > 3) return false;
            if(casilla[f][c].Lapiz().CuentaMarcas() == 3) { cas3 = &casilla[f][c]; cuenta3Candidatos++; }
            if(cuenta3Candidatos > 1) return false;
        }
    }
    if(cuenta3Candidatos == 0) return false; // En ste caso el sudoku estar resuelto
    // Si no hemos retornado false, tenemos un patrn en el que todas las casillas
    // con candidatos tienen dos, menos una que tiene 3.
    // La casilla (f3,c3) es la que tiene tres candidatos, en la fila f3, la columna c3 y
    // en la caja donde est esa casilla hay un candidato que aparece 3 veces, localizarlo.
    for(int v=1; v<=9; v++) {
        if(cas3->Lapiz().ContieneMarca(v) && fila[cas3->Fila()].CuentaCandidato(v) == 3) {
            sprintf(cad, "Todas las casillas tienen dos candidatos menos %c%c", 'A'+cas3->Fila(), '1'+cas3->Columna());
            Log(cad);
            sprintf(cad, "-> Valor final %d en %c%c", v, 'A'+cas3->Fila(), '1'+cas3->Columna());
            Log(cad);
            // Marcar valor final v en cas3
            Asignar(cas3->Fila(), cas3->Columna(), v);
            cas3->Resaltar();
        }
    }
    return true;
}

bool Tablero::BuscaFila(Desnudo des) {
    int c1, c2, ff;
    int nMarcas1, nMarcas2;
    Casilla *cas3 = NULL;
    char cad[100];

    ff = des.Cas(0)->Fila();
    c1 = des.Cas(0)->Columna();
    c2 = des.Cas(1)->Columna();
    for(int f=0; f<9; f++) {
        if(f != ff) {
            nMarcas1 = casilla[f][c1].Lapiz().CuentaMarcas();
            nMarcas2 = casilla[f][c2].Lapiz().CuentaMarcas();

            if(nMarcas1==2 && nMarcas2>=3 &&
               casilla[f][c1].Lapiz() == des.Cas(0)->Lapiz() &&
               (casilla[f][c2].Lapiz() ^ des.Cas(0)->Lapiz()) == des.Cas(0)->Lapiz()) {
                cas3 = &casilla[f][c2];
            } else
            if(nMarcas1>=3 && nMarcas2==2 &&
               casilla[f][c2].Lapiz() == des.Cas(0)->Lapiz() &&
               (casilla[f][c1].Lapiz() ^ des.Cas(0)->Lapiz()) == des.Cas(0)->Lapiz()) {
                cas3 = &casilla[f][c1];
            }
        }
    }

    if(cas3) {
        sprintf(cad, "Solucion unica A %c%c %c%c %c%c %c%c valores %s",
            'A'+ff, '1'+c1, 'A'+ff, '1'+c2, 'A'+cas3->Fila(), '1'+c1, 'A'+cas3->Fila(), '1'+c2,
            des.Cas(0)->Lapiz().Valores());
        Log(cad);
        des.Cas(0)->Amarillo();
        des.Cas(1)->Amarillo();
        if(c1 != cas3->Columna()) casilla[cas3->Fila()][c1].Amarillo();
        if(c2 != cas3->Columna()) casilla[cas3->Fila()][c2].Amarillo();
        casilla[cas3->Fila()][cas3->Columna()].Resaltar();
        for(int v=1; v<=9; v++) {
            if(des.Cas(0)->Lapiz().ContieneMarca(v)) {
                cas3->Lapiz().EliminaMarca(v);
                sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+cas3->Fila(), '1'+cas3->Columna());
                Log(cad);
            }
        }
        return true;
    }
    return false;
}

bool Tablero::BuscaColumna(Desnudo des) {
    int f1, f2, cc;
    int nMarcas1, nMarcas2;
    Casilla *cas3 = NULL;
    char cad[100];

    cc = des.Cas(0)->Columna();
    f1 = des.Cas(0)->Fila();
    f2 = des.Cas(1)->Fila();
    for(int c=0; c<9; c++) {
        if(c != cc) {
            nMarcas1 = casilla[f1][c].Lapiz().CuentaMarcas();
            nMarcas2 = casilla[f1][c].Lapiz().CuentaMarcas();

            if(nMarcas1==2 && nMarcas2>=3 &&
               casilla[f1][c].Lapiz() == des.Cas(0)->Lapiz() &&
               (casilla[f2][c].Lapiz() ^ des.Cas(0)->Lapiz()) == des.Cas(0)->Lapiz()) {
                cas3 = &casilla[f2][c];
            } else
            if(nMarcas1>=3 && nMarcas2==2 &&
               casilla[f2][c].Lapiz() == des.Cas(0)->Lapiz() &&
               (casilla[f1][c].Lapiz() ^ des.Cas(0)->Lapiz()) == des.Cas(0)->Lapiz()) {
                cas3 = &casilla[f1][c];
            }
        }
    }

    if(cas3) {
        sprintf(cad, "Solucion unica A %c%c %c%c %c%c %c%c valores %s",
            'A'+f1, '1'+cc, 'A'+f2, '1'+cc, 'A'+f1, '1'+cas3->Columna(), 'A'+f2, '1'+cas3->Columna(),
            des.Cas(0)->Lapiz().Valores());
        Log(cad);
        des.Cas(0)->Amarillo();
        des.Cas(1)->Amarillo();
        if(f1 != cas3->Fila()) casilla[f1][cas3->Columna()].Amarillo();
        if(f1 != cas3->Fila()) casilla[f2][cas3->Columna()].Amarillo();
        casilla[cas3->Fila()][cas3->Columna()].Resaltar();
        for(int v=1; v<=9; v++) {
            if(des.Cas(0)->Lapiz().ContieneMarca(v)) {
                cas3->Lapiz().EliminaMarca(v);
                sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+cas3->Fila(), '1'+cas3->Columna());
                Log(cad);
            }
        }
        return true;
    }
    return false;
}

bool Tablero::SolucionUnicaA() {
    int ff, cc;

    LimpiaResaltes();
    desnudo.Primero();
    while(desnudo.Actual()) {
        // Si se trata de un par desnudo y las dos casillas estn en la misma caja
        if(desnudo.ValorActual().Par() && desnudo.ValorActual().MismaCaja() != -1) {
            ff = desnudo.ValorActual().MismaFila();
            cc = desnudo.ValorActual().MismaColumna();
            if(-1 != ff) { // El par desnudo est en una fila
                // Buscar en filas
                if(BuscaFila(desnudo.ValorActual())) return true;
            }
            if(-1 != cc) {
                // Buscar en columnas
                if(BuscaColumna(desnudo.ValorActual())) return true;
            }
        }
        desnudo.Siguiente();
    }
    return false;
}

bool Tablero::BuscaFilaTrios(Desnudo des) {
    bool retval=false;
    int c1, c2, ff;
    MarcasLapiz m;
    int v;
    char cad[100];

    ff = des.Cas(0)->Fila();
    c1 = des.Cas(0)->Columna();
    c2 = des.Cas(1)->Columna();
    for(int f=0; f<9; f++) {
        if(f != ff) {
            if(casilla[f][c1].Lapiz().CuentaMarcas()==3 && casilla[f][c2].Lapiz().CuentaMarcas()==3 &&
               casilla[f][c1].Lapiz() == casilla[f][c2].Lapiz() &&
               (casilla[f][c1].Lapiz() ^ des.Cas(0)->Lapiz()) == (casilla[f][c2].Lapiz() ^ des.Cas(0)->Lapiz())) {
                m = casilla[f][c1].Lapiz() - des.Cas(0)->Lapiz(); // Marca extra
                v = m.Valor(); // Valor de la marca extra
                // Eliminar v del resto de la fila f
                for(int c=0; c<9; c++)
                    if(c != c1 && c!= c2) retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                // Si el par desnudo est en la misma caja, tambin se puede eliminar v del resto de la caja del trio
                if(casilla[f][c1].Caja() == casilla[f][c2].Caja()) {
                    int k = casilla[f][c1].Caja();
                    for(int p=0; p<9; p++)
                        if(caja[k].Cas(p)->Fila() != f) retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(v);
                }
                if(retval) {
                    sprintf(cad, "Solucion unica B %c%c %c%c %c%c %c%c valor %d",
                        'A'+ff, '1'+c1, 'A'+ff, '1'+c2, 'A'+f, '1'+c1, 'A'+f, '1'+c2, v);
                    Log(cad);
                    des.Cas(0)->Amarillo();
                    des.Cas(1)->Amarillo();
                    casilla[f][c1].Amarillo();
                    casilla[f][c2].Amarillo();
                    for(int c=0; c<9; c++) {
                        if(c != c1 && c!= c2 && casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                    if(casilla[f][c1].Caja() == casilla[f][c2].Caja()) {
                        int k = casilla[f][c1].Caja();
                        for(int p=0; p<9; p++) {
                            if(caja[k].Cas(p)->Fila() != f && caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                                sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                                Log(cad);
                                caja[k].Cas(p)->Resaltar();
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaColumnaTrios(Desnudo des) {
    bool retval=false;
    int f1, f2, cc;
    MarcasLapiz m;
    int v;
    char cad[100];

    cc = des.Cas(0)->Columna();
    f1 = des.Cas(0)->Fila();
    f2 = des.Cas(1)->Fila();
    for(int c=0; c<9; c++) {
        if(c != cc) {
            if(casilla[f1][c].Lapiz().CuentaMarcas()==3 && casilla[f2][c].Lapiz().CuentaMarcas()==3 &&
               casilla[f1][c].Lapiz() == casilla[f2][c].Lapiz() &&
               (casilla[f1][c].Lapiz() ^ des.Cas(0)->Lapiz()) == (casilla[f2][c].Lapiz() ^ des.Cas(0)->Lapiz())) {
                m = casilla[f1][c].Lapiz() - des.Cas(0)->Lapiz(); // Marca extra
                v = m.Valor(); // Valor de la marca extra
                // Eliminar v del resto de la columna c
                for(int f=0; f<9; f++)
                    if(f != f1 && f!= f2) retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                // Si el par desnudo est en la misma caja, tambin se puede eliminar v del resto de la caja del trio
                if(casilla[f1][c].Caja() == casilla[f2][c].Caja()) {
                    int k = casilla[f1][c].Caja();
                    for(int p=0; p<9; p++)
                        if(caja[k].Cas(p)->Columna() != c) retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(v);
                }
                if(retval) {
                    sprintf(cad, "Solucion unica B %c%c %c%c %c%c %c%c valor %d",
                        'A'+f1, '1'+cc, 'A'+f2, '1'+cc, 'A'+f1, '1'+c, 'A'+f2, '1'+c, v);
                    Log(cad);
                    des.Cas(0)->Amarillo();
                    des.Cas(1)->Amarillo();
                    casilla[f1][c].Amarillo();
                    casilla[f2][c].Amarillo();
                    for(int f=0; f<9; f++) {
                        if(f != f1 && f!= f2 && casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                    if(casilla[f1][c].Caja() == casilla[f2][c].Caja()) {
                        int k = casilla[f1][c].Caja();
                        for(int p=0; p<9; p++) {
                            if(caja[k].Cas(p)->Columna() != c && caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                                sprintf(cad, "-> Eliminar valor %d de %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                                Log(cad);
                                caja[k].Cas(p)->Resaltar();
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::SolucionUnicaB() {
    int ff, cc;

    LimpiaResaltes();
    desnudo.Primero();
    while(desnudo.Actual()) {
        ff = desnudo.ValorActual().MismaFila();
        cc = desnudo.ValorActual().MismaColumna();
        if(-1 != ff && desnudo.ValorActual().Par()) { // El par desnudo est en una fila
            // Buscar en filas
            if(BuscaFilaTrios(desnudo.ValorActual())) return true;
        }
        if(-1 != cc && desnudo.ValorActual().Par()) {
            if(BuscaColumnaTrios(desnudo.ValorActual())) return true;
        }
        desnudo.Siguiente();
    }
    return false;
}

// Para cada casilla con tres candidatos buscamos dos alas compatibles con un patrn XYZ-Wing
bool Tablero::ProcesaXYZWing() {
    MarcasLapiz m[3];
    int i, j, k, v;
    int pos[3][3]; // array con zonas, indice 1 (f, c, k) e indice 2 con las marcas,

    LimpiaResaltes();
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() == 3) {
                // Calcular las tres combinaciones de marcas de lpiz
                // empezamos con las tres, y eliminamos una distinta en cada una
                m[0] = m[1] = m[2] = casilla[f][c].Lapiz();
                i=0;
                for(int v=1; v<=9; v++) {
                    if(m[i].ContieneMarca(v)) { m[i++].EliminaMarca(v); }
                }
                k = casilla[f][c].Caja();
                // Buscar en misma fila, columna y caja casillas con dos candidatos coincidentes
                for(i = 0; i<3; i++) {
                    pos[0][i] = fila[f].BuscarMarcas(m[i]);
                    pos[1][i] = columna[c].BuscarMarcas(m[i]);
                    pos[2][i] = caja[k].BuscarMarcas(m[i]);
                }
                // Solo puede haber un patron XYZ-Wing si se ha detectado una casilla con marcas en la caja
                for(i=0; i<3; i++) {
                    if(pos[2][i] != -1) { // Tenemos el par m[i] en la pos[2][i] de la caja
                        for(j=0; j<3; j++) { // Buscar las otras dos parejas en fila o columna
                            if(pos[0][j] != -1 && i!=j) { // Tenemos el par m[j] en la columna pos[0][j]
                                v = (m[i] ^ m[j]).Valor(); // Valor comn de m[i] y m[j]
                                if(f!=caja[k].Cas(pos[2][i])->Fila() && casilla[f][pos[0][j]].Caja() != k)
                                   if(ProcesarXYZFila(k, f, c, v, pos[0][j], pos[2][i])) return true;
                            }
                            if(pos[1][j] != -1 && i!=j) { // Tenemos el par m[j] en la fila pos[1][j]
                                v = (m[i] ^ m[j]).Valor(); // Valor comn de m[i] y m[j]
                                if(c != caja[k].Cas(pos[2][i])->Columna() && casilla[pos[1][j]][c].Caja() != k)
                                    if(ProcesarXYZColumna(k, f, c, v, pos[1][j], pos[2][i])) return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::ProcesarXYZFila(int k, int fil, int col, int v, int c, int p) {
    bool retval=false;
    char cad[100];

    // Hay que eliminar el candidato v en la fila fil de la caja k,
    // excepto en la columna en que est el pivote.
    for(int p=0; p<9; p++) {
        if(caja[k].Cas(p)->Fila() == fil && caja[k].Cas(p)->Columna() != col)
            retval = caja[k].Cas(p)->Lapiz().ContieneMarca(v);
    }
    if(retval) {
        sprintf(cad, "XYZ-Wing en %c%c -> %c%c %c%c", 'A'+fil, '1'+col,
            'A'+fil, '1'+c, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
        Log(cad);
        casilla[fil][col].Naranja();
        casilla[fil][c].Amarillo();
        caja[k].Cas(p)->Amarillo();
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Fila() == fil && caja[k].Cas(p)->Columna() != col)
                if(caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                    sprintf(cad, "-> Eliminar %d en %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                    Log(cad);
                    caja[k].Cas(p)->Resaltar();
                }
        }
    }
    return retval;
}

bool Tablero::ProcesarXYZColumna(int k, int fil, int col, int v, int f, int p) {
    bool retval=false;
    char cad[100];

    // Hay que eliminar el candidato v en la columna col de la caja k,
    // excepto en la fila en que est el pivote.
    for(int p=0; p<9; p++) {
        if(caja[k].Cas(p)->Columna() == col && caja[k].Cas(p)->Fila() != fil)
            retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(v);
    }
    if(retval) {
        sprintf(cad, "XYZ-Wing en %c%c -> %c%c %c%c", 'A'+fil, '1'+col,
            'A'+f, '1'+col, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
        Log(cad);
        casilla[fil][col].Naranja();
        casilla[f][col].Amarillo();
        caja[k].Cas(p)->Amarillo();
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Columna() == col && caja[k].Cas(p)->Fila() != fil)
                if(caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                    sprintf(cad, "-> Eliminar %d en %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                    Log(cad);
                    caja[k].Cas(p)->Resaltar();
                }
        }
    }
    return retval;
}

bool Tablero::ProcesaSashimi() {
    LimpiaResaltes();
    if(BuscaSashimiFilas()) return true;
    if(BuscaSashimiColumnas()) return true;
    return false;
}

bool Tablero::BuscaSashimiFilas() {
    Pareja par;
    int c, c1, c2;
    int k, cuenta;

    for(int v=1; v<=9; v++) {
        for(int f1=0; f1<8; f1++) {
            par = fila[f1].DobleCandidato(v);
            if(par.Cas(0) && !par.MismaCaja() ) {
                c1 = par.Cas(0)->Columna();
                c2 = par.Cas(1)->Columna();
                for(int f2=0; f2<9; f2++) {
                    if(f2 != f1) {
                        c = -1;
                        if(casilla[f2][c1].Lapiz().ContieneMarca(v) && casilla[f2][c2].Valor()) c = c2;
                        if(casilla[f2][c2].Lapiz().ContieneMarca(v) && casilla[f2][c1].Valor()) c = c1;
                        if(c != -1) {
                            k = casilla[f2][c].Caja();
                            // Contar casillas con candidato v en fila f2 y caja k
                            cuenta=0;
                            for(int p=0; p<9; p++) {
                                if(caja[k].Cas(p)->Columna() == c && caja[k].Cas(p)->Lapiz().ContieneMarca(v)) cuenta++;
                            }
                            if(cuenta==2 && columna[c].CuentaCandidato(v) == 3) {
                                // Posible, depende de si hay candidatos a eliminar:
                                if(ProcesarSashimiFila(f1, f2, c1, c2, v, c)) return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::BuscaSashimiColumnas() {
    Pareja par;
    int f, f1, f2;
    int k, cuenta;

    for(int v=1; v<=9; v++) {
        for(int c1=0; c1<8; c1++) {
            par = columna[c1].DobleCandidato(v);
            if(par.Cas(0) && !par.MismaCaja()) {
                f1 = par.Cas(0)->Fila();
                f2 = par.Cas(1)->Fila();
                for(int c2=0; c2<9; c2++) {
                    if(c2!= c1) {
                        f = -1;
                        if(casilla[f1][c2].Lapiz().ContieneMarca(v) && casilla[f2][c2].Valor()) f = f2;
                        if(casilla[f2][c2].Lapiz().ContieneMarca(v) && casilla[f1][c2].Valor()) f = f1;
                        if(f != -1) {
                            k = casilla[f][c2].Caja();
                            // Contar casillas con candidato v en fila f2 y caja k
                            cuenta=0;
                            for(int p=0; p<9; p++) {
                                if(caja[k].Cas(p)->Fila() == f && caja[k].Cas(p)->Lapiz().ContieneMarca(v)) cuenta++;
                            }
                            if(cuenta == 2 && fila[c2].CuentaCandidato(v) == 3) {
                                // Posible, depende de si hay candidatos a eliminar:
                                if(ProcesarSashimiColumna(f1, f2, c1, c2, v, f)) return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::ProcesarSashimiFila(int f1, int f2, int c1, int c2, int v, int c) {
    int retval = false;
    int k = casilla[f2][c].Caja();
    int f3[2], i=0;
    char cad[100];

    for(int p=0; p<9; p++) {
        if(caja[k].Cas(p)->Fila() == f2 && caja[k].Cas(p)->Columna() != c) retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(v);
    }
    if(retval) {
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Columna() == c && caja[k].Cas(p)->Lapiz().ContieneMarca(v)) f3[i++] = caja[k].Cas(p)->Fila();
        }
        if(c==c2)
            sprintf(cad, "Sashimi X-Wing F %c%c %c%c %c%c %c%c %c%c, valor %d",
                'A'+f1, '1'+c1, 'A'+f1, '1'+c2, 'A'+f2, '1'+c1, 'A'+f3[0], '1'+c, 'A'+f3[1], '1'+c, v);
        else
            sprintf(cad, "Sashimi X-Wing F %c%c %c%c %c%c %c%c %c%c, valor %d",
                'A'+f1, '1'+c1, 'A'+f1, '1'+c2, 'A'+f3[0], '1'+c, 'A'+f3[1], '1'+c, 'A'+f2, '1'+c2, v);
        Log(cad);
        casilla[f1][c1].Amarillo();
        casilla[f1][c2].Amarillo();
        if(c==c2)
            casilla[f2][c1].Amarillo();
        else
            casilla[f2][c2].Amarillo();
        casilla[f3[0]][c].Amarillo();
        casilla[f3[1]][c].Amarillo();
        casilla[f2][c].Naranja();
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Fila() == f2 && caja[k].Cas(p)->Columna() != c)
                if(caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                    sprintf(cad, "-> eliminar %d en %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                    Log(cad);
                    caja[k].Cas(p)->Resaltar();
                }
        }
    }
    return retval;
}

bool Tablero::ProcesarSashimiColumna(int f1, int f2, int c1, int c2, int v, int f) {
    int retval = false;
    int k = casilla[f][c2].Caja();
    int c3[2], i=0;
    char cad[100];

    for(int p=0; p<9; p++) {
        if(caja[k].Cas(p)->Columna() == c2 && caja[k].Cas(p)->Fila() != f) retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(v);
    }
    if(retval) {
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Fila() == f && caja[k].Cas(p)->Lapiz().ContieneMarca(v)) c3[i++] = caja[k].Cas(p)->Columna();
        }
        if(f==f2)
            sprintf(cad, "Sashimi X-Wing C %c%c %c%c %c%c %c%c %c%c, valor %d",
                'A'+f1, '1'+c1, 'A'+f1, '1'+c2, 'A'+f1, '1'+c2, 'A'+f, '1'+c3[0], 'A'+f, '1'+c3[1], v);
        else
            sprintf(cad, "Sashimi X-Wing C %c%c %c%c %c%c %c%c %c%c, valor %d",
                'A'+f1, '1'+c1, 'A'+f1, '1'+c2, 'A'+f, '1'+c3[0], 'A'+f, '1'+c3[1], 'A'+f2, '1'+c2, v);
        Log(cad);
        casilla[f1][c1].Amarillo();
        casilla[f1][c2].Amarillo();
        if(f==f2)
            casilla[f1][c2].Amarillo();
        else
            casilla[f2][c2].Amarillo();
        casilla[f][c3[0]].Amarillo();
        casilla[f][c3[1]].Amarillo();
        casilla[f][c2].Naranja();
        for(int p=0; p<9; p++) {
            if(caja[k].Cas(p)->Columna() == c2 && caja[k].Cas(p)->Fila() != f)
                if(caja[k].Cas(p)->Lapiz().EliminaMarca(v)) {
                    sprintf(cad, "-> eliminar %d en %c%c", v, 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                    Log(cad);
                    caja[k].Cas(p)->Resaltar();
                }
        }
    }
    return retval;
}

bool Tablero::ProcesaSkyscraper() {
    LimpiaResaltes();
    if(BuscaSkyscraperFilas()) return true;
    if(BuscaSkyscraperColumnas()) return true;
    return false;
}

bool Tablero::BuscaSkyscraperFilas() {
    Pareja par1, par2;

    for(int v=1; v<=9; v++) {
        for(int f1=0; f1<8; f1++) {
            par1 = fila[f1].DobleCandidato(v);
            // La pareja de candidatos no puede estar en la misma caja
            if(par1.Cas(0) && par1.Cas(0)->Caja() != par1.Cas(1)->Caja()) {
                for(int f2=f1+1; f2<9; f2++) {
                    par2 = fila[f2].DobleCandidato(v);
                    if(par2.Cas(0)) {
                        // Las casillas de cada pareja deben estar en las misma columna de cajas, y en distintas cajas
                        if(par1.Cas(0)->Caja()%3 == par2.Cas(0)->Caja()%3 && par1.Cas(1)->Caja()%3 == par2.Cas(1)->Caja()%3 &&
                            par1.Cas(0)->Caja() != par2.Cas(0)->Caja()){
                            // Segunda condicin:
                            // Una de las casillas de cada pareja debe estar en la misma columna
                            if(par1.Cas(0)->Columna() == par2.Cas(0)->Columna() && par1.Cas(1)->Columna() != par2.Cas(1)->Columna()) {
                                if(ProcesarSkyscraperFila(par1, par2, 1, v)) return true;
                            }
                            if(par1.Cas(0)->Columna() != par2.Cas(0)->Columna() && par1.Cas(1)->Columna() == par2.Cas(1)->Columna()) {
                                if(ProcesarSkyscraperFila(par1, par2, 0, v)) return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::BuscaSkyscraperColumnas() {
    Pareja par1, par2;

    for(int v=1; v<=9; v++) {
        for(int c1=0; c1<8; c1++) {
            par1 = columna[c1].DobleCandidato(v);
            // La pareja de candidatos no puede estar en la misma caja
            if(par1.Cas(0) && par1.Cas(0)->Caja() != par1.Cas(1)->Caja()) {
                for(int c2=c1+1; c2<9; c2++) {
                    par2 = columna[c2].DobleCandidato(v);
                    if(par2.Cas(0)) {
                        // Las casillas de cada pareja deben estar en las misma fila de cajas, y en distintas cajas
                        if(par1.Cas(0)->Caja()/3 == par2.Cas(0)->Caja()/3 && par1.Cas(1)->Caja()/3 == par2.Cas(1)->Caja()/3 &&
                            par1.Cas(0)->Caja() != par2.Cas(0)->Caja()){
                            // Segunda condicin:
                            // Una de las casillas de cada pareja debe estar en la misma columna
                            if(par1.Cas(0)->Fila() == par2.Cas(0)->Fila() && par1.Cas(1)->Fila() != par2.Cas(1)->Fila()) {
                                if(ProcesarSkyscraperColumna(par1, par2, 1, v)) return true;
                            }
                            if(par1.Cas(0)->Fila() != par2.Cas(0)->Fila() && par1.Cas(1)->Fila() == par2.Cas(1)->Fila()) {
                                if(ProcesarSkyscraperColumna(par1, par2, 0, v)) return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::ProcesarSkyscraperFila(Pareja &par1, Pareja &par2, int np, int v) {
    bool retval = false;
    int k1, k2;
    char cad[100];

    k1 = par1.Cas(np)->Caja();
    k2 = par2.Cas(np)->Caja();
    for(int f=0; f<9; f++) {
        if(casilla[f][par1.Cas(np)->Columna()].Caja() == k2) {
            retval |= casilla[f][par1.Cas(np)->Columna()].Lapiz().ContieneMarca(v);
        }
        if(casilla[f][par2.Cas(np)->Columna()].Caja() == k1) {
            retval |= casilla[f][par2.Cas(np)->Columna()].Lapiz().ContieneMarca(v);
        }
    }
    if(retval) {
        sprintf(cad, "Skyscraper F %c%c %c%c %c%c %c%c, %d",
            'A'+par1.Cas(0)->Fila(), '1'+par1.Cas(0)->Columna(), 'A'+par1.Cas(1)->Fila(), '1'+par1.Cas(1)->Columna(),
            'A'+par2.Cas(0)->Fila(), '1'+par2.Cas(0)->Columna(), 'A'+par2.Cas(1)->Fila(), '1'+par2.Cas(1)->Columna(), v);
        Log(cad);
        casilla[par1.Cas(0)->Fila()][par1.Cas(0)->Columna()].Amarillo();
        casilla[par1.Cas(1)->Fila()][par1.Cas(1)->Columna()].Amarillo();
        casilla[par2.Cas(0)->Fila()][par2.Cas(0)->Columna()].Amarillo();
        casilla[par2.Cas(1)->Fila()][par2.Cas(1)->Columna()].Amarillo();
        for(int f=0; f<9; f++) {
            if(casilla[f][par1.Cas(np)->Columna()].Caja() == k2) {
                if(casilla[f][par1.Cas(np)->Columna()].Lapiz().EliminaMarca(v))
                    casilla[f][par1.Cas(np)->Columna()].Resaltar();
            }
            if(casilla[f][par2.Cas(np)->Columna()].Caja() == k1) {
                if(casilla[f][par2.Cas(np)->Columna()].Lapiz().EliminaMarca(v))
                    casilla[f][par2.Cas(np)->Columna()].Resaltar();
            }
        }
    }
    return retval;
}

bool Tablero::ProcesarSkyscraperColumna(Pareja &par1, Pareja &par2, int np, int v) {
    bool retval = false;
    int k1, k2;
    char cad[100];

    k1 = par1.Cas(np)->Caja();
    k2 = par2.Cas(np)->Caja();
    for(int c=0; c<9; c++) {
        if(casilla[par1.Cas(np)->Fila()][c].Caja() == k2) {
            retval |= casilla[par1.Cas(np)->Fila()][c].Lapiz().ContieneMarca(v);
        }
        if(casilla[par2.Cas(np)->Fila()][c].Caja() == k1) {
            retval |= casilla[par2.Cas(np)->Fila()][c].Lapiz().ContieneMarca(v);
        }
    }
    if(retval) {
        sprintf(cad, "Skyscraper C %c%c %c%c %c%c %c%c, %d",
            'A'+par1.Cas(0)->Fila(), '1'+par1.Cas(0)->Columna(), 'A'+par1.Cas(1)->Fila(), '1'+par1.Cas(1)->Columna(),
            'A'+par2.Cas(0)->Fila(), '1'+par2.Cas(0)->Columna(), 'A'+par2.Cas(1)->Fila(), '1'+par2.Cas(1)->Columna(), v);
        Log(cad);
        casilla[par1.Cas(0)->Fila()][par1.Cas(0)->Columna()].Amarillo();
        casilla[par1.Cas(1)->Fila()][par1.Cas(1)->Columna()].Amarillo();
        casilla[par2.Cas(0)->Fila()][par2.Cas(0)->Columna()].Amarillo();
        casilla[par2.Cas(1)->Fila()][par2.Cas(1)->Columna()].Amarillo();
        for(int c=0; c<9; c++) {
            if(casilla[par1.Cas(np)->Fila()][c].Caja() == k2) {
                if(casilla[par1.Cas(np)->Fila()][c].Lapiz().EliminaMarca(v))
                    casilla[par1.Cas(np)->Fila()][c].Resaltar();
            }
            if(casilla[par2.Cas(np)->Fila()][c].Caja() == k1) {
                if(casilla[par2.Cas(np)->Fila()][c].Lapiz().EliminaMarca(v))
                    casilla[par2.Cas(np)->Fila()][c].Resaltar();
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaRectangulosVacios() {
    int rv;
    int f, c;
    int f2, c2;
    Pareja par;
    char cad[100];

    LimpiaResaltes();
    for(int v=1; v<=9; v++) {
        // En cada caja buscar patrones de rectngulos vacos
        for(int k=0; k<9; k++) {
            rv = caja[k].RectanguloVacio(v);
            if(-1 != rv) {
                f = 3*(k/3) + rv/3;
                c = 3*(k%3) + rv%3;
                // Buscar par puntero en columnas con un candiadto en fila f
                for(int c1=0; c1<9; c1++) {
                    par = columna[c1].DobleCandidato(v);
                    if(par.Cas(0) && // Si encuentra un par
                       !par.MismaCaja() && // El par no est en la misma caja
                       (par.Cas(0)->Caja() != k) && (par.Cas(1)->Caja() != k) &&// Y est en una fila de cajas distinta a la del rectngulo
                       (par.Cas(0)->Fila() == f || par.Cas(1)->Fila() == f)) { // Y una de las casillas coincide con la fila en el rectngulo
                        if(par.Cas(0)->Fila() == f) f2 = par.Cas(1)->Fila();
                        else f2 = par.Cas(0)->Fila();
                        if(casilla[f2][c].Lapiz().ContieneMarca(v)) {
                            sprintf(cad, "Vacio: %c%c en %d -> col %c valor %d (%c%c)", 'A'+f, '1'+c, 1+k, '1'+c1, v, 'A'+f2, '1'+c);
                            Log(cad);
                            for(int p=0; p<9; p++) if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) caja[k].Cas(p)->Amarillo();
                            par.Cas(0)->Naranja();
                            par.Cas(1)->Naranja();
                            casilla[f2][c].Lapiz().EliminaMarca(v);
                            casilla[f2][c].Resaltar();
                            sprintf(cad, "->Elimina %d en %c%c", v, 'A'+f2, '1'+c);
                            Log(cad);
                            return true;
                        }
                    }
                }
                // Buscar par puntero en filas con un candiadto en fila c
                for(int f1=0; f1<9; f1++) {
                    par = fila[f1].DobleCandidato(v);
                    if(par.Cas(0) && !par.MismaCaja() &&
                       (par.Cas(0)->Caja() != k) && (par.Cas(1)->Caja() != k) &&
                       (par.Cas(0)->Columna() == c || par.Cas(1)->Columna() == c)) {
                        if(par.Cas(0)->Columna() == c) c2 = par.Cas(1)->Columna();
                        else c2 = par.Cas(0)->Columna();
                        if(casilla[f][c2].Lapiz().ContieneMarca(v)) {
                            sprintf(cad, "Vacio: %c%c en %d -> fil %c valor %d (%c%c)", 'A'+f, '1'+c, 1+k, 'A'+f1, v, 'A'+f, '1'+c2);
                            Log(cad);
                            for(int p=0; p<9; p++) if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) caja[k].Cas(p)->Amarillo();
                            par.Cas(0)->Naranja();
                            par.Cas(1)->Naranja();
                            casilla[f][c2].Lapiz().EliminaMarca(v);
                            casilla[f][c2].Resaltar();
                            sprintf(cad, "->Elimina %d en %c%c", v, 'A'+f, '1'+c2);
                            Log(cad);
                            return true;
                        }
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::BuscaJellyFishFilas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;

    for(int f1=0; f1<6; f1++) {
        // Buscamos en las primeras 6 filas en las que no aparezca v
        // y est como candidato en 2, 3 o 4 columnas:
        if(!fila[f1].Presente(v) && fila[f1].CuentaCandidato(v) <= 4) {
            // Ahora buscamos las filas f1+1 a septima otra que tenga 2, 3 o 4 candidatos v.
            for(int f2= f1+1; f2<7; f2++) {
                if(!fila[f2].Presente(v) && fila[f2].CuentaCandidato(v) <= 4) {
                    // Si entre las dos filas, el candidato aparece en 3 o 4 columnas, buscamos otra fila
                    CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v);
                    if(CasillasColumnas.CuentaCasillas() <= 4) {
                        // Ahora buscamos las filas f2+1 a octava otra que tenga 2, 3 o 4 candidatos v en las
                        // mismas columnas que las anteriores.
                        for(int f3=f2+1; f3<8; f3++) {
                            if(!fila[f3].Presente(v) && fila[f3].CuentaCandidato(v) <= 4) {
                                CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v)+fila[f3].Marcas(v);
                                if(CasillasColumnas.CuentaCasillas() <= 4) {
                                    // Ahora buscamos las filas f3+1 a novena otra que tenga 2, 3 o 4 candidatos v en las
                                    // mismas columnas que las anteriores.
                                    for(int f4=f3+1; f4<9; f4++) {
                                        if(!fila[f4].Presente(v) && fila[f4].CuentaCandidato(v) <= 4) {
                                            CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v)+fila[f3].Marcas(v)+fila[f4].Marcas(v);
                                            if(CasillasColumnas.CuentaCasillas() == 4) {
                                                // Tenemos una medusa.
                                                CasillasFilas.AsignaCasilla(f1);
                                                CasillasFilas.AsignaCasilla(f2);
                                                CasillasFilas.AsignaCasilla(f3);
                                                CasillasFilas.AsignaCasilla(f4);
                                                if(ProcesaJellyFishFilas(CasillasFilas, CasillasColumnas, v)) return true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaJellyFishColumnas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;

    for(int c1=0; c1<6; c1++) {
        // Buscamos en las primeras 6 columnas en las que no aparezca v
        // y est como candidato en 2 o 3 columnas:
        if(!columna[c1].Presente(v) && columna[c1].CuentaCandidato(v) <= 4) {
            // Ahora buscamos las columnas c1+1 a septima otra que tenga 2, 3 o 4 candidatos v.
            for(int c2= c1+1; c2<7; c2++) {
                if(!columna[c2].Presente(v) && columna[c2].CuentaCandidato(v) <= 4) {
                    // Si entre las dos columnas, el candidato aparece en tres filas, buscamos otra columna
                    CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v);
                    if(CasillasFilas.CuentaCasillas() <= 4) {
                        // Ahora buscamos las columnas c2+1 a octava otra que tenga 2, 3 o 4 candidatos v en las
                        // mismas filas que las anteriores.
                        for(int c3=c2+1; c3<8; c3++) {
                            if(!columna[c3].Presente(v) && columna[c3].CuentaCandidato(v) <= 4) {
                                CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v)+columna[c3].Marcas(v);
                                if(CasillasFilas.CuentaCasillas() <= 4) {
                                    // Ahora buscamos las columnas c3+1 a novena otra que tenga 2, 3 o 4 candidatos v en las
                                    // mismas columnas que las anteriores.
                                    for(int c4=c3+1; c4<9; c4++) {
                                        if(!columna[c4].Presente(v) && columna[c4].CuentaCandidato(v) <= 4) {
                                            CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v)+columna[c3].Marcas(v)+columna[c4].Marcas(v);
                                            if(CasillasFilas.CuentaCasillas() == 4) {
                                                // Tenemos una medusa.
                                                CasillasColumnas.AsignaCasilla(c1);
                                                CasillasColumnas.AsignaCasilla(c2);
                                                CasillasColumnas.AsignaCasilla(c3);
                                                CasillasColumnas.AsignaCasilla(c4);
                                                if(ProcesaJellyFishColumnas(CasillasFilas, CasillasColumnas, v)) return true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaJellyFishFilas(MarcasCasillas fils, MarcasCasillas cols, int v) {
    bool retval = false;
    char cad[100];
    int ff[4], i=0;
    int cc[4], j=0;

    for(int f=0; f<9; f++) {
        if(!fils.ContieneCasilla(f)) {
            for(int c=0; c<9; c++) {
                if(cols.ContieneCasilla(c)) {
                    retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                }
            }
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(fils.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(cols.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Medusa F: fila [%c,%c,%c,%c] col [%c,%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], 'A'+ff[3], '1'+cc[0], '1'+cc[1], '1'+cc[2], '1'+cc[3], v);
        Log(cad);
        for(i=0; i<4; i++) {
            for(j=0; j<4; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }
        if(casilla[ff[0]])
        for(int f=0; f<9; f++) {
            if(!fils.ContieneCasilla(f)) {
                for(int c=0; c<9; c++) {
                    if(cols.ContieneCasilla(c)) {
                        if(casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Elimina candidato %d en %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaJellyFishColumnas(MarcasCasillas fils, MarcasCasillas cols, int v) {
    bool retval = false;
    char cad[100];
    int ff[4], i=0;
    int cc[4], j=0;

    for(int c=0; c<9; c++) {
        if(!cols.ContieneCasilla(c)) {
            for(int f=0; f<9; f++) {
                if(fils.ContieneCasilla(f)) {
                    retval |= casilla[f][c].Lapiz().ContieneMarca(v);
                }
            }
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(fils.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(cols.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Medusa C: fila [%c,%c,%c,%c] col [%c,%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], 'A'+ff[3], '1'+cc[0], '1'+cc[1], '1'+cc[2], '1'+cc[3], v);
        Log(cad);
        for(i=0; i<4; i++) {
            for(j=0; j<4; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }
        if(casilla[cc[0]])
        for(int c=0; c<9; c++) {
            if(!cols.ContieneCasilla(c)) {
                for(int f=0; f<9; f++) {
                    if(fils.ContieneCasilla(f)) {
                        if(casilla[f][c].Lapiz().EliminaMarca(v)) {
                            sprintf(cad, "-> Elimina candidato %d en %c%c", v, 'A'+f, '1'+c);
                            Log(cad);
                            casilla[f][c].Resaltar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaJellyFish() {
    bool retval=false;

    LimpiaResaltes();
    for(int v=1; v<=9; v++) {
        retval |= BuscaJellyFishFilas(v);
        retval |= BuscaJellyFishColumnas(v);
    }
    return retval;
}

bool Tablero::StringKite() {
    Pareja par;
    Pareja parF, parC;

    for(int v=1; v<=9; v++) {
        for(int k=0; k<9; k++) {
            par = caja[k].DobleCandidato(v);
            if(par.Cas(0) && !par.MismaFila() && !par.MismaColumna()) {
                // Buscar par en columna par.Cas(0) y en fila par.Cas(1)
                parC = columna[par.Cas(0)->Columna()].DobleCandidato(v);
                parF = fila[par.Cas(1)->Fila()].DobleCandidato(v);
                if(parC.Cas(0) && parF.Cas(0) && ProcesarStringKite(par, parF, parC, v)) return true;

                // Buscar par en fila par.Cas(0) y en columna par.Cas(1)
                parC = columna[par.Cas(1)->Columna()].DobleCandidato(v);
                parF = fila[par.Cas(0)->Fila()].DobleCandidato(v);
                if(parC.Cas(0) && parF.Cas(0) && ProcesarStringKite(par, parF, parC, v)) return true;
            }
        }
    }
    return false;
}

bool Tablero::ProcesarStringKite(Pareja &par, Pareja &parF, Pareja &parC, int v) {
    int f, c;
    char cad[100];

    if(par.Cas(0) == parC.Cas(0) || par.Cas(1) == parC.Cas(0))
        f = parC.Cas(1)->Fila();
    else
        f = parC.Cas(0)->Fila();
    if(par.Cas(0) == parF.Cas(0) || par.Cas(1) == parF.Cas(0))
        c = parF.Cas(1)->Columna();
    else
        c = parF.Cas(0)->Columna();
    if(casilla[f][c].Lapiz().ContieneMarca(v)) {
        sprintf(cad, "SringKite caja %c valor %d", '1'+par.Cas(0)->Caja(), v);
        Log(cad);
        parC.Cas(0)->Amarillo();
        parC.Cas(1)->Amarillo();
        parF.Cas(0)->Amarillo();
        parF.Cas(1)->Amarillo();
        sprintf(cad, "-> Elimina %d de %c%c", v, 'A'+f, '1'+c);
        Log(cad);
        casilla[f][c].Resaltar();
        casilla[f][c].Lapiz().EliminaMarca(v);
        return true;
    }
    return false;
}

bool Tablero::SwordFishAlas() {
    bool retval=false;

    LimpiaResaltes();
    for(int v=1; v<=9; v++) {
        retval |= BuscaSwordFishAlasFilas(v);
        retval |= BuscaSwordFishAlasColumnas(v);
    }
    return retval;
}

bool Tablero::BuscaSwordFishAlasFilas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;
    MarcasCasillas CasillasAlas;
    int caj;

    for(int f1=0; f1<8; f1++) {
        // Buscamos en las primeras 8 filas en las que no aparezca v
        // y est como candidato en 2, 3 columnas:
        if(!fila[f1].Presente(v) && fila[f1].CuentaCandidato(v) <= 3) {
            // Ahora buscamos las filas f1+1 a novena otra que tenga 2, 3 candidatos v.
            for(int f2= f1+1; f2<9; f2++) {
                if(!fila[f2].Presente(v) && fila[f2].CuentaCandidato(v) <= 3) {
                    // Si entre las dos filas, el candidato aparece en 2, 3 columnas, buscamos otra fila
                    // que complete un patrn pez espada con alas:
                    CasillasColumnas = fila[f1].Marcas(v)+fila[f2].Marcas(v);
                    if(CasillasColumnas.CuentaCasillas() == 3) {
                        // Ahora buscamos en todas las filas menos f1 y f2 otra que tenga al menos 3 candidatos v
                        for(int f3=0; f3<9; f3++) {
                            if(f3 != f1 && f3 != f2 && !fila[f3].Presente(v) &&
                               fila[f3].CuentaCandidato(v) >= 3 &&
                               (fila[f3].Marcas(v) & CasillasColumnas).CuentaCasillas() >= 2) {
                                CasillasAlas = fila[f3].Marcas(v) & ~CasillasColumnas;
                                caj = CasillasAlas.Cajas();
                                if(((caj & CasillasColumnas.Cajas()) == caj) && (caj == 1 || caj == 2 || caj == 4)) {
                                    // CasillasColumnas tiene las columnas de casillas con el candidato
                                    CasillasFilas.Borra();
                                    CasillasFilas.AsignaCasilla(f1);
                                    CasillasFilas.AsignaCasilla(f2);
                                    CasillasFilas.AsignaCasilla(f3);
                                    if(ProcesaSwordFishAlasFilas(CasillasFilas, CasillasColumnas, CasillasAlas, f3, v)) return true;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaSwordFishAlasColumnas(int v) {
    bool retval=false;
    MarcasCasillas CasillasFilas;
    MarcasCasillas CasillasColumnas;
    MarcasCasillas CasillasAlas;
    int caj;

    for(int c1=0; c1<8; c1++) {
        // Buscamos en las primeras 8 columnas en las que no aparezca v
        // y est como candidato en 2, 3 filas:
        if(!columna[c1].Presente(v) && columna[c1].CuentaCandidato(v) <= 3) {
            // Ahora buscamos las columnas c1+1 a novena otra que tenga 2, 3 candidatos v.
            for(int c2= c1+1; c2<9; c2++) {
                if(!columna[c2].Presente(v) && columna[c2].CuentaCandidato(v) <= 3) {
                    // Si entre las dos columnas, el candidato aparece en 2, 3 filas, buscamos otra columna
                    // que complete un patrn pez espada con alas:
                    CasillasFilas = columna[c1].Marcas(v)+columna[c2].Marcas(v);
                    if(CasillasFilas.CuentaCasillas() == 3) {
                        // Ahora buscamos en todas las Columnas menos c1 y c2 otra que tenga al menos 3 candidatos v
                        for(int c3=0; c3<9; c3++) {
                            if(c3 != c1 && c3 != c2 && !columna[c3].Presente(v) &&
                               columna[c3].CuentaCandidato(v) >= 3 &&
                               (columna[c3].Marcas(v) & CasillasFilas).CuentaCasillas() >= 2) {
                                CasillasAlas = columna[c3].Marcas(v) & ~CasillasFilas;
                                caj = CasillasAlas.Cajas();
                                if(((caj & CasillasFilas.Cajas()) == caj) && (caj == 1 || caj == 2 || caj == 4)) {
                                    // CasillasFilas tiene las filas de casillas con el candidato
                                    CasillasColumnas.Borra();
                                    CasillasColumnas.AsignaCasilla(c1);
                                    CasillasColumnas.AsignaCasilla(c2);
                                    CasillasColumnas.AsignaCasilla(c3);
                                    if(ProcesaSwordFishAlasColumnas(CasillasFilas, CasillasColumnas, CasillasAlas, c3, v)) return true;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaSwordFishAlasFilas(MarcasCasillas &CasillasFilas,
                                        MarcasCasillas &CasillasColumnas,
                                        MarcasCasillas &CasillasAlas, int f3, int v) {
    bool retval = false;
    int ca, c2;
    MarcasCasillas ck;
    int caj;
    int ff[3], cc[3], i=0, j=0;
    char cad[100];

    caj = CasillasAlas.Cajas();
    ca = CasillasAlas.Casilla();
    ck = CasillasColumnas;
    ck.MascaraCaja(caj >> 1);
    c2 = ck.Casilla();
    // Eliminar candidatos en columna de candidato que comparte caja con [f3][c]
    // menos en filas f1, f2 y f3
    for(int f=0; f<9; f++) {
        if(!CasillasFilas.ContieneCasilla(f) && casilla[f][ca].Caja() == casilla[f3][c2].Caja()) {
            retval |= casilla[f][c2].Lapiz().ContieneMarca(v);
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(CasillasFilas.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(CasillasColumnas.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Pez espada Alas F: fila [%c,%c,%c] col [%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], '1'+cc[0], '1'+cc[1], '1'+cc[2], v);
        Log(cad);
        for(i=0; i<3; i++) {
            for(j=0; j<3; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }

        for(int f=0; f<9; f++) {
            if(!CasillasFilas.ContieneCasilla(f) && casilla[f][ca].Caja() == casilla[f3][c2].Caja()) {
                if(casilla[f][c2].Lapiz().EliminaMarca(v)) {
                    casilla[f][c2].Resaltar();
                    sprintf(cad, "-> Eliminar %d en %c%c", v, 'A'+f, '1'+c2);
                    Log(cad);
                }
            }
        }
    }
    return retval;
}

bool Tablero::ProcesaSwordFishAlasColumnas(MarcasCasillas &CasillasFilas,
                                           MarcasCasillas &CasillasColumnas,
                                           MarcasCasillas &CasillasAlas, int c3, int v) {
    bool retval = false;
    int fa, f2;
    MarcasCasillas fk;
    int caj;
    int ff[3], cc[3], i=0, j=0;
    char cad[100];

    caj = CasillasAlas.Cajas();
    fa = CasillasAlas.Casilla();
    fk = CasillasFilas;
    fk.MascaraCaja(caj >> 1);
    f2 = fk.Casilla();
    // Eliminar candidatos en fila de candidato que comparte caja con [f][c3]
    // menos en columnas c1, c2 y c3
    for(int c=0; c<9; c++) {
        if(!CasillasColumnas.ContieneCasilla(c) && casilla[fa][c].Caja() == casilla[f2][c3].Caja()) {
            retval |= casilla[f2][c].Lapiz().ContieneMarca(v);
        }
    }

    if(retval) {
        for(int f=0; f<9; f++) if(CasillasFilas.ContieneCasilla(f)) ff[i++] = f;
        for(int c=0; c<9; c++) if(CasillasColumnas.ContieneCasilla(c)) cc[j++] = c;
        sprintf(cad, "Pez espada Alas C: fila [%c,%c,%c] col [%c,%c,%c] valor %d",
                'A'+ff[0], 'A'+ff[1], 'A'+ff[2], '1'+cc[0], '1'+cc[1], '1'+cc[2], v);
        Log(cad);
        for(i=0; i<3; i++) {
            for(j=0; j<3; j++) {
                if(casilla[ff[i]][cc[j]].Lapiz().ContieneMarca(v))
                    casilla[ff[i]][cc[j]].Amarillo();
            }
        }

        for(int c=0; c<9; c++) {
            if(!CasillasColumnas.ContieneCasilla(c) && casilla[fa][c].Caja() == casilla[f2][c3].Caja()) {
                if(casilla[f2][c].Lapiz().EliminaMarca(v)) {
                    casilla[f2][c].Resaltar();
                    sprintf(cad, "-> Eliminar %d en %c%c", v, 'A'+f2, '1'+c);
                    Log(cad);
                }
            }
        }
    }
    return retval;
}

void Tablero::Mostrar() {
    int f, c;
    WORD color1, color2;

    color1 = 15;
    color2 = 3;

    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(f%3==0) {
                Texto(f*4,  c*4, "+", color1);
                Texto(f*4,  c*4+1, "---+", color1);
            } else {
                Texto(f*4,  c*4, "+", color2);
                Texto(f*4,  c*4+1, "---+", color2);
            }
            if(c%3==0) {
                Texto(f*4+1,c*4, "|", color1);
                Texto(f*4+2,c*4, "|", color1);
                Texto(f*4+3,c*4, "|", color1);
            } else {
                Texto(f*4+1,c*4, "|", color2);
                Texto(f*4+1,c*4+4, "|", color2);
                Texto(f*4+2,c*4, "|", color2);
                Texto(f*4+2,c*4+4, "|", color2);
                Texto(f*4+3,c*4, "|", color2);
                Texto(f*4+3,c*4+4, "|", color2);
            }
            Texto(f*4+1,c*4+4, "|", color1);
            Texto(f*4+2,c*4+4, "|", color1);
            Texto(f*4+3,c*4+4, "|", color1);
        }
    }
    for(c=0; c<9; c++) {
        Texto(36,  c*4, "+", color1);
        Texto(36,  c*4+1, "---+", color1);
    }

    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].Mostrar();
        }
    }
    Texto(55, 0, " ", color1);
}

