// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaracin de la clase Tablero

#ifndef SUDOKU_TABLERO
#define SUDOKU_TABLERO

#include <fstream>
#include "casilla.h"
#include "fila.h"
#include "columna.h"
#include "caja.h"
#include "lista.h"
#include "desnudo.h"
#include "puntero.h"
#include "grafo.h"
#include "marcascasillas.h"

class Tablero {
    private:
    Casilla casilla[9][9];
    Fila fila[9];
    Columna columna[9];
    Caja caja[9];
    int cuenta;
    std::ofstream flog;
    Lista<Desnudo> desnudo;
    Lista<Puntero> puntero;

    bool BuscaParesDesnudosFila(int f);
    bool BuscaParesDesnudosColumna(int c);
    bool BuscaParesDesnudosCaja(int c);

    bool ProcesaParesDesnudosFila(int f);
    bool ProcesaParesDesnudosColumna(int c);
    bool ProcesaParesDesnudosCaja(int c);

    bool ProcesaParesDesnudosFilaFase2(int f);
    bool ProcesaParesDesnudosColumnaFase2(int c);
    bool ProcesaParesDesnudosCajaFase2(int c);

    bool BuscaPunterosFila(int f, int v);
    bool BuscaPunterosColumna(int c, int v);
    bool BuscaPunterosCaja(int k, int v);

    bool ProcesaPunterosFila(int f);
    bool ProcesaPunterosColumna(int c);
    bool ProcesaPunterosCaja(int k);

    bool ProcesaGrafoFila(int grupo);
    bool ProcesaGrafoColumna(int grupo);
    bool ProcesaGrafoCaja(int grupo);

    bool ProcesarXYWingCaso1(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso2(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso3(Casilla *pivote, Casilla *ala1, Casilla *ala2);

    bool ProcesaXWingFila(Pareja a, Pareja b, int v);
    bool ProcesaXWingColumna(Pareja a, Pareja b, int v);

    bool BuscaXWingAlasFilas(int v);
    bool BuscaXWingAlasColumnas(int v);
    bool ProcesaXWingAlasFilas(Pareja par, Casilla *f2, int v);
    bool ProcesaXWingAlasColumnas(Pareja par, Casilla *c2, int v);

    bool BuscaSwordFishFilas(int v);
    bool BuscaSwordFishColumnas(int v);
    bool ProcesaSwordFishFilas(MarcasCasillas, MarcasCasillas, int v);
    bool ProcesaSwordFishColumnas(MarcasCasillas, MarcasCasillas, int v);

    bool BuscaFila(Desnudo des);
    bool BuscaColumna(Desnudo des);

    bool BuscaFilaTrios(Desnudo des);
    bool BuscaColumnaTrios(Desnudo des);

    bool ProcesarXYZFila(int k, int fil, int col, int v, int c, int p);
    bool ProcesarXYZColumna(int k, int fil, int col, int v, int f, int p);

    bool BuscaSashimiFilas();
    bool BuscaSashimiColumnas();
    bool ProcesarSashimiFila(int f1, int f2, int c1, int c2, int v, int c);
    bool ProcesarSashimiColumna(int f1, int f2, int c1, int c2, int v, int c);

    bool BuscaSkyscraperFilas();
    bool BuscaSkyscraperColumnas();
    bool ProcesarSkyscraperFila(Pareja &par1, Pareja &par2, int np, int v);
    bool ProcesarSkyscraperColumna(Pareja &par1, Pareja &par2, int np, int v);

    bool BuscaJellyFishFilas(int v);
    bool BuscaJellyFishColumnas(int v);
    bool ProcesaJellyFishFilas(MarcasCasillas, MarcasCasillas, int v);
    bool ProcesaJellyFishColumnas(MarcasCasillas, MarcasCasillas, int v);

    bool ProcesarStringKite(Pareja &par, Pareja &parF, Pareja &parC, int v);

    bool BuscaSwordFishAlasFilas(int v);
    bool BuscaSwordFishAlasColumnas(int v);
    bool ProcesaSwordFishAlasFilas(MarcasCasillas &CasillasFilas,
                                   MarcasCasillas &CasillasColumnas,
                                   MarcasCasillas &CasillasAlas,
                                   int f3, int v);
    bool ProcesaSwordFishAlasColumnas(MarcasCasillas &CasillasFilas,
                                      MarcasCasillas &CasillasColumnas,
                                      MarcasCasillas &CasillasAlas,
                                      int c3, int v);

    public:
    Tablero(char* t);
    ~Tablero() { flog.close(); }
    void Log(char *cad);
    Casilla &GetCasilla(int f, int c) { return casilla[f][c]; }
    const int Cuenta() { return cuenta; }
    void Mostrar();
    bool Soluciones();
    bool Valido(int f, int c, int v);
    void Asignar(int f, int c, int v);

    // Resolucin:
    void LimpiaResaltes();
    void LimpiaCriba();
    void Criba(int v);
    bool ProcesarCriba(int v);
    bool CandidatoUnico();
    bool ProcesarCribaSolitarios(int v);
    bool BuscaParesDesnudos();
    bool ProcesaParesDesnudos();
    // Resolucion FASE 2
    void PonerMarcasLapiz();
    bool CasillasConUnaMarcaLapiz();
    bool UnicaCasillaConCandidato();
    bool VerificarFinales();
    bool ProcesaParesDesnudosFase2();
    bool BuscaPunteros();
    bool ProcesaPunteros();
    bool ProcesaGrafo(int grupo);
    bool ProcesaXYWing();
    bool ProcesaXWing();
    bool ProcesaXWingAlas();
    bool ProcesaSwordFish();
    bool TodasConDosCandidatosMenosUna();
    bool SolucionUnicaA();
    bool SolucionUnicaB();
    bool ProcesaXYZWing();
    bool ProcesaSashimi();
    bool ProcesaSkyscraper();
    bool ProcesaRectangulosVacios();
    bool ProcesaJellyFish();
    bool StringKite();
    bool SwordFishAlas();

    Lista<Desnudo> *ListaDesnudo() { return &desnudo; }
    Lista<Puntero> *ListaPuntero() { return &puntero; }
};
#endif // SUDOKU_TABLERO
